"""
Resolver for male-specific job and occupation category labels.
This module provides functionality to translate categories combining jobs,
nationalities, and descriptors for men into idiomatic Arabic.
"""

import functools
import logging
import re

from ...helps import len_print
from ...new.handle_suffixes import resolve_sport_category_suffix_with_mapping
from ...translations import (  # all_country_with_nat_ar,
    RELIGIOUS_KEYS_PP,
    All_Nat,
    all_country_with_nat,
    countries_en_as_nationality_keys,
    jobs_mens_data,
)
from ...translations_formats import MultiDataFormatterBaseV2, format_multi_data_v2
from ..nats_as_country_names import nats_keys_as_country_names
from .utils import fix_keys, nat_and_gender_keys, one_Keys_more_2

logger = logging.getLogger(__name__)
countries_en_keys = [x.get("en") for x in all_country_with_nat.values() if x.get("en")]

jobs_mens_data_f = dict(jobs_mens_data.items())
# jobs_mens_data_f.update({x: v["males"] for x, v in RELIGIOUS_KEYS_PP.items() if v.get("males")})

REGEX_THE = re.compile(r"\b(the)\b", re.I)

keys_not_jobs = [
    "women",
    "men",
]

Mens_prefix: dict[str, str] = {
    # "men": "رجال",
    # "expatriate male": "ذكور مغتربون",
    # "expatriate men's": "رجال مغتربون",
    # "male": "ذكور",
    # "male child": "أطفال ذكور",
    "amputee": "مبتورو أحد الأطراف",
    "blind": "مكفوفون",
    "child": "أطفال",
    "children": "أطفال",
    "deaf": "صم",
    "deafblind": "صم ومكفوفون",
    "expatriate": "مغتربون",
    "latin": "لاتينيون",
    "lgbt": "مثليون",
    "murdered": "قتلوا",
    "mythological": "أسطوريون",
    "nautical": "بحريون",
    "renaissance": "عصر النهضة",
    "romantic": "رومانسيون",
    "sunni muslim": "مسلمون سنة",
}

genders_keys_new_under_test: dict[str, str] = {
    "kidnapped": "مختطفون",
    "fictional": "خياليون",
    "disabled": "معاقون",
    "contemporary": "معاصرون",
    "latin": "لاتينيون",
    "child": "أطفال",
    "political": "سياسيون",
    "religious": "دينيون",
    # "military": "عسكريون", # NOTE: cause errors like: "British military": "بريطانيون عسكريون",
}

genders_keys: dict[str, str] = {
    "assassinated": "مغتالون",
    # "male deaf": "صم ذكور",
    "blind": "مكفوفون",
    "abolitionists": "مناهضون للعبودية",
    "deaf": "صم",
    "executed": "أعدموا",
    "executed abroad": "أعدموا في الخارج",
    "deafblind": "صم ومكفوفون",
    "killed-in-action": "قتلوا في عمليات قتالية",
    "killed in action": "قتلوا في عمليات قتالية",
    "murdered abroad": "قتلوا في الخارج",
}

genders_keys.update(genders_keys_new_under_test)


def is_false_key(key: str, value: str) -> bool:
    if ("mens" in key.lower() or "men's" in key.lower()) and "رجالية" in value:
        return True

    if key in genders_keys:  # NOTE: under test
        return True

    if RELIGIOUS_KEYS_PP.get(key):
        return True

    if key in keys_not_jobs:
        return True

    not_in_keys = [
        "expatriate",
        "immigrants",
    ]

    # if any(word in key for word in not_in_keys) and not
    if any(word in key for word in not_in_keys):
        return True

    return False


@functools.lru_cache(maxsize=1)
def _load_formatted_data() -> dict:
    formatted_data_jobs_with_nat = {
        "political office-holders": "أصحاب مناصب سياسية",
        "{en_nat} political office-holders": "أصحاب مناصب سياسية {males}",
        # base keys
        "{en_nat} muslim scholars-of-islam": "باحثون عن الإسلام مسلمون {males}",
        "{en_nat} sunni muslim scholars-of-islam": "باحثون عن الإسلام مسلمون سنة {males}",
        "{en_nat} sunni muslim scholars of islam": "باحثون عن الإسلام مسلمون سنة {males}",
        "{en_nat} contemporary classical musicians": "موسيقيون كلاسيكيون معاصرون {males}",
        "{en_nat} contemporary classical composers": "ملحنون كلاسيكيون معاصرون {males}",
        "{en_nat}": "{males}",
        "{en_nat} muslims": "{males} مسلمون",
        "{en_nat} muslim": "{males} مسلمون",
        # "{en_nat} people": "أعلام {males}",
        # "{en_nat} people": "{males}",
        "{en_nat}-american coaches of canadian-football": "مدربو كرة قدم كندية أمريكيون {males}",
        "{en_nat} coaches of canadian-football": "مدربو كرة قدم كندية {males}",
        "{en_nat}-american": "{males} أمريكيون",
        "{en_nat} eugenicists": "علماء {males} متخصصون في تحسين النسل",
        "{en_nat} politicians who committed suicide": "سياسيون {males} أقدموا على الانتحار",
        "{en_nat} contemporary artists": "فنانون {males} معاصرون",
        # [Category:Turkish expatriate sports-people] : "تصنيف:رياضيون أتراك مغتربون"
        "{en_nat} expatriate {en_job}": "{ar_job} {males} مغتربون",
        # "Category:Pakistani expatriate male actors": "تصنيف:ممثلون ذكور باكستانيون مغتربون",
        "{en_nat} expatriate male {en_job}": "{ar_job} ذكور {males} مغتربون",
        # [Category:Turkish immigrants sports-people] : "تصنيف:رياضيون أتراك مهاجرون"
        "{en_nat} immigrants {en_job}": "{ar_job} {males} مهاجرون",
        "{en_nat} films people": "أعلام أفلام {males}",
        "{en_nat} film people": "أعلام أفلام {males}",
        "male {en_nat}": "{males} ذكور",
        "men {en_nat}": "{males}",  # رجال
        "mens {en_nat}": "{males}",  # رجال
        # emigrants keys
        # "{en_nat} emigrants": "{ar_job} مهاجرون",
        "{en_nat} emigrants {en_job}": "{ar_job} {males} مهاجرون",
        "emigrants {en_nat} {en_job}": "{ar_job} مهاجرون",
        # "spouses of {en_nat} politicians": "قرينات سياسيون {males}",
        "spouses of {en_nat}": "قرينات {males}",
        "spouses of {en_nat} {en_job}": "قرينات {ar_job} {males}",
    }

    # { "{en_nat} male emigrants": "{males} مهاجرون ذكور", "{en_nat} emigrants male": "{males} مهاجرون ذكور", "male {en_nat} emigrants": "{males} مهاجرون ذكور" }
    formatted_data_jobs_with_nat.update(nat_and_gender_keys("{en_nat}", "emigrants", "male", "{males} مهاجرون ذكور"))
    formatted_data_jobs_with_nat.update(nat_and_gender_keys("{en_nat}", "expatriate", "male", "{males} مغتربون ذكور"))

    formatted_data_jobs = {
        # base keys
        "{en_job}": "{ar_job}",
        "{en_job} people": "أعلام {ar_job}",
        "male {en_job}": "{ar_job} ذكور",
        "men {en_job}": "{ar_job}",  # رجال
        "mens {en_job}": "{ar_job}",  # رجال
        # expatriate keys
        "expatriate {en_job}": "{ar_job} مغتربون",
        "expatriate male {en_job}": "{ar_job} ذكور مغتربون",
        # emigrants keys
        "emigrants {en_job}": "{ar_job} مهاجرون",
    }
    formatted_data_jobs.update(nat_and_gender_keys("{en_job}", "emigrants", "male", "{ar_job} مهاجرون ذكور"))
    formatted_data_jobs.update(nat_and_gender_keys("{en_job}", "expatriate", "male", "{ar_job} مغتربون ذكور"))

    formatted_data = dict(formatted_data_jobs)
    formatted_data.update(
        {
            f"{{en_nat}} {x}": f"{v} {{males}}"
            for x, v in formatted_data_jobs.items()
            if "{en_nat}" not in x and "{males}" not in v
        }
    )

    # formatted_data.update({
    #     f"{{en_nat}}-american {x}" : f"{v} أمريكيون {{males}}" for x, v in formatted_data_jobs.items()
    # })

    for x, v in genders_keys.items():
        keys_more = one_Keys_more_2(
            x,
            v,
            en_nat_key="{en_nat}",
            en_job_key="{en_job}",
            ar_nat_key="{males}",
            ar_job_key="{ar_job}",
            add_women=False,
        )
        formatted_data.update(keys_more)

    formatted_data.update(formatted_data_jobs_with_nat)
    formatted_data.update(
        {
            "fictional {en_nat} jews": "{males} يهود خياليون",
            "ancient {en_nat}": "{males} قدماء",
            "ancient {en_job}": "{ar_job} قدماء",
            "military {en_job}": "{ar_job} عسكريون",
            "{en_nat} emigrants": "{males} مهاجرون",
            "fictional {en_nat} religious workers": "عمال دينيون {males} خياليون",
            "{en_nat} religious workers": "عمال دينيون {males}",
            # "{en_nat} saints": "{males} قديسون",
            "{en_nat} anti-communists": "{males} مناهضون للشيوعية",
            "{en_nat} disability rights activists": "{males} ناشطون في حقوق الإعاقة",
            # "executed {en_nat}": "{males} أعدموا",
            # "{en_nat} executed": "{males} أعدموا",
            # "{en_nat} executed abroad": "{males} أعدموا في الخارج",
            "{en_nat} eugenicists": "علماء {males} متخصصون في تحسين النسل",
            "{en_nat} politicians who committed suicide": "سياسيون {males} أقدموا على الانتحار",
            "{en_nat} contemporary artists": "فنانون {males} معاصرون",
            "{en_nat} scholars of islam": "{males} باحثون عن الإسلام",
            "{en_nat} womens rights activists": "{males} ناشطون في حقوق المرأة",
            "{en_nat} businesspeople": "شخصيات أعمال {female}",
        }
    )

    NAT_BEFORE_OCC_BASE = [
        "murdered abroad",
        "contemporary",
        "tour de france stage winners",
        "deafblind",
        "deaf",
        "blind",
        "jews",
        # "women's rights activists",
        "female rights activists",
        "human rights activists",
        "imprisoned",
        "imprisoned abroad",
        "conservationists",
        "expatriate",
        "defectors",
        "scholars of islam",
        "scholars-of-islam",
        "amputees",
        "executed abroad",
        "emigrants",
    ]
    for x in NAT_BEFORE_OCC_BASE:
        if jobs_mens_data_f.get(x):
            formatted_data[f"{{en_nat}} {x}"] = f"{{males}} {jobs_mens_data_f[x]}"

    formatted_data_final = {x.replace("'", ""): v for x, v in formatted_data.items()}
    return formatted_data_final


@functools.lru_cache(maxsize=1)
def _load_jobs_data() -> dict[str, str]:
    # all keys without any word from not_in_keys
    data = {x: {"ar_job": v} for x, v in jobs_mens_data_f.items() if not is_false_key(x, v)}
    len_diff = len(set(jobs_mens_data_f.keys()) - set(data.keys()))

    if len_diff:
        logger.warning(f" mens before fix: {len(data):,}, is_false_key diff: {len_diff:,}")

    data.update(
        {
            "philosophers and theologians": {"ar_job": "فلاسفة ولاهوتيون"},
        }
    )

    data = {x.replace("'", "").replace("australian rules", "australian-rules"): v for x, v in data.items()}
    return data


@functools.lru_cache(maxsize=1)
def _load_nat_data() -> dict[str, str]:
    # nats_data: dict[str, str] = {x: v for x, v in all_country_with_nat_ar.items()}  # 342
    nats_data: dict[str, str] = dict(All_Nat.items())  # 342

    nats_data.update(dict(nats_keys_as_country_names.items()))

    nats_data.update(
        {
            "jewish american": {
                "male": "أمريكي يهودي",
                "males": "أمريكيون يهود",
                "female": "أمريكية يهودية",
                "females": "أمريكيات يهوديات",
                "the_male": "الأمريكي اليهودي",
                "the_female": "الأمريكية اليهودية",
                "en": "",
                "ar": "",
            }
        }
    )

    nats_data = {x.replace("'", ""): v for x, v in nats_data.items()}
    return nats_data


@functools.lru_cache(maxsize=1)
def load_bot() -> MultiDataFormatterBaseV2:
    jobs_data_enhanced = _load_jobs_data()
    logger.debug(f"jobs_data_enhanced mens: {len(jobs_data_enhanced):,}")

    formatted_data = _load_formatted_data()

    logger.debug(f"_load_formatted_data mens: {len(formatted_data):,}")

    nats_data = _load_nat_data()
    return format_multi_data_v2(
        formatted_data=formatted_data,
        data_list=nats_data,
        key_placeholder="{en_nat}",
        # value_placeholder="{males}",
        data_list2=jobs_data_enhanced,
        key2_placeholder="{en_job}",
        # value2_placeholder="{ar_job}",
        text_after=" people",
        text_before="the ",
        use_other_formatted_data=True,
        search_first_part=True,
    )


@functools.lru_cache(maxsize=10000)
def _mens_resolver_labels(category: str) -> str:
    logger.debug(f"<<yellow>> start {category=}")
    category = fix_keys(category).replace("australian rules", "australian-rules")

    _bot = load_bot()
    result = _bot.search_all_category(category)

    logger.info(f"<<yellow>> end {category=}, {result=}")
    return result


religious_data = {x: f"{{}} {v['males']}" for x, v in RELIGIOUS_KEYS_PP.items() if v.get("males")}

label_mappings_ends = dict(
    sorted(
        religious_data.items(),
        key=lambda k: (-k[0].count(" "), -len(k[0])),
    )
)


@functools.lru_cache(maxsize=10000)
def mens_resolver_labels(category: str) -> str:
    logger.debug(f"<<yellow>> start {category=}")
    category = fix_keys(category).replace("australian rules", "australian-rules")

    if category in countries_en_as_nationality_keys or category in countries_en_keys:
        logger.info(f"<<yellow>> skip : {category=}, [result=]")
        return ""

    result = _mens_resolver_labels(category) or resolve_sport_category_suffix_with_mapping(
        category=category,
        data=label_mappings_ends,
        callback=_mens_resolver_labels,
        format_key="{}",
    )
    logger.info(f"<<yellow>> end {category=}, {result=}")
    return result


len_print.data_len(
    "mens.py",
    {
        "mens_formatted_data": _load_formatted_data(),
        "mens_jobs_data": _load_jobs_data(),
    },
)



"""
Package for resolving film and television related categories.
This package provides specialized resolvers for film genres, production years,
and television series, often combined with geographic elements.
"""

import functools
import logging
import re

from ...translations import TELEVISION_KEYS, Films_key_CAO
from .resolve_films_labels import get_films_key_tyty_new
from .resolve_films_labels_and_time import get_films_key_tyty_new_and_time

logger = logging.getLogger(__name__)


def legacy_label_check(normalized_category: str) -> str:
    """
    Resolve legacy labels or return a numeric category string.

    Parameters:
        normalized_category (str): Normalized category text to check.

    Returns:
        str: Arabic label for known legacy terms, the numeric string if the input is all digits, or an empty string if no match.
    """
    label = ""
    if re.match(r"^\d+$", normalized_category.strip()):
        label = normalized_category.strip()

    if normalized_category == "people":
        label = "أشخاص"
    return label


@functools.lru_cache(maxsize=10000)
def main_films_resolvers(normalized_category) -> str:
    """
    Resolve a film nationalities label from a category string.

    Normalizes the input by trimming whitespace, converting to lowercase, and removing a leading "category:" prefix, then queries a sequence of film-label resolvers and returns the first non-empty result.

    Parameters:
        normalized_category (str): Category text to resolve; may include a leading "category:" prefix.

    Returns:
        str: The resolved label if any resolver finds a match, otherwise an empty string.
    """
    normalized_category = normalized_category.strip().lower().replace("category:", "")

    logger.debug("--" * 20)
    logger.debug(f"<><><><><><> <<green>> {normalized_category=}")

    resolved_label = (
        legacy_label_check(normalized_category)
        or get_films_key_tyty_new_and_time(normalized_category)
        or TELEVISION_KEYS.get(normalized_category)
        or Films_key_CAO.get(normalized_category)
        or get_films_key_tyty_new(normalized_category)
        or ""
    )

    logger.info(f"<<yellow>> end {normalized_category=}, {resolved_label=}")
    return resolved_label


__all__ = [
    "main_films_resolvers",
    "get_films_key_tyty_new",
    "get_films_key_tyty_new_and_time",
]



#
import pytest

from ArWikiCats import resolve_label_ar
from utils.dump_runner import make_dump_test_name_data

religions_data = {
    "Albanian Sunni Muslim scholars of Islam": "باحثون عن الإسلام مسلمون سنة ألبان",
    "Algerian Sunni Muslim scholars of Islam": "باحثون عن الإسلام مسلمون سنة جزائريون",
    "American Sunni Muslim scholars of Islam": "باحثون عن الإسلام مسلمون سنة أمريكيون",
    "Arab Sunni Muslim scholars of Islam": "باحثون عن الإسلام مسلمون سنة عرب",
    "Australian Sunni Muslim scholars of Islam": "باحثون عن الإسلام مسلمون سنة أستراليون",
    "Bangladeshi Sunni Muslim scholars of Islam": "باحثون عن الإسلام مسلمون سنة بنغلاديشيون",
    "Bosnian Sunni Muslim scholars of Islam": "باحثون عن الإسلام مسلمون سنة بوسنيون",
    "Egyptian Sunni Muslim scholars of Islam": "باحثون عن الإسلام مسلمون سنة مصريون",
    "English Sunni Muslim scholars of Islam": "باحثون عن الإسلام مسلمون سنة إنجليز",
    "Filipino Sunni Muslim scholars of Islam": "باحثون عن الإسلام مسلمون سنة فلبينيون",
    "Indian Sunni Muslim scholars of Islam": "باحثون عن الإسلام مسلمون سنة هنود",
    "Indonesian Sunni Muslim scholars of Islam": "باحثون عن الإسلام مسلمون سنة إندونيسيون",
    "Iraqi Sunni Muslim scholars of Islam": "باحثون عن الإسلام مسلمون سنة عراقيون",
    "Mauritanian Sunni Muslim scholars of Islam": "باحثون عن الإسلام مسلمون سنة موريتانيون",
    "Moroccan Sunni Muslim scholars of Islam": "باحثون عن الإسلام مسلمون سنة مغاربة",
    "Pakistani Sunni Muslim scholars of Islam": "باحثون عن الإسلام مسلمون سنة باكستانيون",
    "Palestinian Sunni Muslim scholars of Islam": "باحثون عن الإسلام مسلمون سنة فلسطينيون",
    "Persian Sunni Muslim scholars of Islam": "باحثون عن الإسلام مسلمون سنة فرس",
    "Saudi Arabian Sunni Muslim scholars of Islam": "باحثون عن الإسلام مسلمون سنة سعوديون",
    "Somalian Sunni Muslim scholars of Islam": "باحثون عن الإسلام مسلمون سنة صوماليون",
    "Syrian Sunni Muslim scholars of Islam": "باحثون عن الإسلام مسلمون سنة سوريون",
    "Turkish Sunni Muslim scholars of Islam": "باحثون عن الإسلام مسلمون سنة أتراك",
    "Yemeni Sunni Muslim scholars of Islam": "باحثون عن الإسلام مسلمون سنة يمنيون",
    "13th-century indian muslims": "هنود مسلمون في القرن 13",
    "14th-century indian muslims": "هنود مسلمون في القرن 14",
    "15th-century indian muslims": "هنود مسلمون في القرن 15",
    "16th-century indian muslims": "هنود مسلمون في القرن 16",
    "17th-century indian muslims": "هنود مسلمون في القرن 17",
    "18th-century indian muslims": "هنود مسلمون في القرن 18",
    "19th-century indian muslims": "هنود مسلمون في القرن 19",
    "20th-century indian muslims": "هنود مسلمون في القرن 20",
    "21st-century indian muslims": "هنود مسلمون في القرن 21",
    "afghan muslims": "أفغان مسلمون",
    "african muslims": "أفارقة مسلمون",
    "african-american muslims": "أمريكيون أفارقة مسلمون",
    "albanian muslims": "ألبان مسلمون",
    "algerian muslims": "جزائريون مسلمون",
    "american muslims": "أمريكيون مسلمون",
    "arab muslims": "عرب مسلمون",
    "argentine muslims": "أرجنتينيون مسلمون",
    "armenian muslims": "أرمن مسلمون",
    "asian muslims": "آسيويون مسلمون",
    "australian muslims": "أستراليون مسلمون",
    "austrian muslims": "نمساويون مسلمون",
    "azerbaijani muslims": "أذربيجانيون مسلمون",
    "bahraini muslims": "بحرينيون مسلمون",
    "bangladeshi muslims": "بنغلاديشيون مسلمون",
    "basque muslims": "باسكيون مسلمون",
    "belarusian muslims": "بيلاروسيون مسلمون",
    "belgian muslims": "بلجيكيون مسلمون",
    "bengali muslims": "بنغاليون مسلمون",
    "beninese muslims": "بنينيون مسلمون",
    "bosnia and herzegovina muslims": "بوسنيون مسلمون",
    "bosnian muslim collaborators with nazi germany": "بوسنيون مسلمون متعاونون مع ألمانيا النازية",
    "bosnian muslims from ottoman empire": "بوسنيون مسلمون من الدولة العثمانية",
    "bosnian muslims": "بوسنيون مسلمون",
    "brazilian muslims": "برازيليون مسلمون",
    "british muslims": "بريطانيون مسلمون",
    "bruneian muslims": "برونيون مسلمون",
    "bulgarian muslims": "بلغاريون مسلمون",
    "burkinabe muslims": "بوركينابيون مسلمون",
    "burmese muslims": "بورميون مسلمون",
    "burundian muslims": "بورونديون مسلمون",
    "byzantine muslims": "بيزنطيون مسلمون",
    "cambodian muslims": "كمبوديون مسلمون",
    "cameroonian muslims": "كاميرونيون مسلمون",
    "canadian muslims": "كنديون مسلمون",
    "central african republic muslims": "أفارقة أوسطيون مسلمون",
    "central asian muslims": "آسيويون أوسطيون مسلمون",
    "chadian muslims": "تشاديون مسلمون",
    "chinese muslims": "صينيون مسلمون",
    "colombian muslims": "كولومبيون مسلمون",
    "comorian muslims": "قمريون مسلمون",
    "croatian muslims": "كروات مسلمون",
    "cuban muslims": "كوبيون مسلمون",
    "cypriot muslims": "قبرصيون مسلمون",
    "czech muslims": "تشيكيون مسلمون",
    "danish muslims": "دنماركيون مسلمون",
    "democratic republic of congo muslims": "كونغويون ديمقراطيون مسلمون",
    "dutch muslims": "هولنديون مسلمون",
    "east timorese muslims": "تيموريون شرقيون مسلمون",
    "egyptian muslims": "مصريون مسلمون",
    "emirati muslims": "إماراتيون مسلمون",
    "english muslims": "إنجليز مسلمون",
    "eritrean muslims": "إريتريون مسلمون",
    "ethiopian muslims": "إثيوبيون مسلمون",
    "european muslims": "أوروبيون مسلمون",
    "fijian muslims": "فيجيون مسلمون",
    "filipino muslims": "فلبينيون مسلمون",
    "finnish muslims": "فنلنديون مسلمون",
    "french muslims": "فرنسيون مسلمون",
    "gabonese muslims": "غابونيون مسلمون",
    "gambian muslims": "غامبيون مسلمون",
    "georgian muslims": "جورجيون مسلمون",
    "german muslims": "ألمان مسلمون",
    "ghanaian muslims": "غانيون مسلمون",
    "greek muslims": "يونانيون مسلمون",
    "guinean muslims": "غينيون مسلمون",
    "gujarati muslims": "غجراتيون مسلمون",
    "guyanese muslims": "غيانيون مسلمون",
    "haitian muslims": "هايتيون مسلمون",
    "hong kong muslims": "هونغ كونغيون مسلمون",
    "hungarian muslims": "مجريون مسلمون",
    "icelandic muslims": "آيسلنديون مسلمون",
    "indian muslims by century": "هنود مسلمون حسب القرن",
    "indian muslims": "هنود مسلمون",
    "indonesian muslims": "إندونيسيون مسلمون",
    "iranian muslims": "إيرانيون مسلمون",
    "iraqi muslims": "عراقيون مسلمون",
    "irish muslims": "أيرلنديون مسلمون",
    "israeli muslims": "إسرائيليون مسلمون",
    "italian muslims": "إيطاليون مسلمون",
    "ivorian muslims": "إيفواريون مسلمون",
    "jamaican muslims": "جامايكيون مسلمون",
    "japanese muslims": "يابانيون مسلمون",
    "jordanian muslims": "أردنيون مسلمون",
    "kashmiri muslims": "كشميريون مسلمون",
    "kazakhstani muslims": "كازاخستانيون مسلمون",
    "kenyan muslims": "كينيون مسلمون",
    "korean muslims": "كوريون مسلمون",
    "kosovan muslims": "كوسوفيون مسلمون",
    "kurdish muslims": "أكراد مسلمون",
    "kuwaiti muslims": "كويتيون مسلمون",
    "kyrgyzstani muslims": "قيرغيزستانيون مسلمون",
    "latvian muslims": "لاتفيون مسلمون",
    "lebanese muslims": "لبنانيون مسلمون",
    "liberian muslims": "ليبيريون مسلمون",
    "libyan muslims": "ليبيون مسلمون",
    "macedonian muslims": "مقدونيون مسلمون",
    "malagasy muslims": "مدغشقريون مسلمون",
    "malawian muslims": "ملاويون مسلمون",
    "malaysian muslims": "ماليزيون مسلمون",
    "maldivian muslims": "مالديفيون مسلمون",
    "malian muslims": "ماليون مسلمون",
    "mass murder of sri lankan muslims": "قتل جماعي في سريلانكيون مسلمون",
    "massacres of bengali muslims": "مذابح في بنغاليون مسلمون",
    "mauritanian muslims": "موريتانيون مسلمون",
    "mauritian muslims": "موريشيوسيون مسلمون",
    "mexican muslims": "مكسيكيون مسلمون",
    "middle eastern muslims": "شرقيون أوسطيون مسلمون",
    "moldovan muslims": "مولدوفيون مسلمون",
    "mongolian muslims": "منغوليون مسلمون",
    "montenegrin muslims": "مونتينيغريون مسلمون",
    "moroccan muslims": "مغاربة مسلمون",
    "mozambican muslims": "موزمبيقيون مسلمون",
    "namibian muslims": "ناميبيون مسلمون",
    "nepalese muslims": "نيباليون مسلمون",
    "nigerian muslims": "نيجيريون مسلمون",
    "nigerien muslims": "نيجريون مسلمون",
    "north american muslims": "أمريكيون شماليون مسلمون",
    "norwegian muslims": "نرويجيون مسلمون",
    "oceanian muslims": "أوقيانوسيون مسلمون",
    "omani muslims": "عمانيون مسلمون",
    "pakistani muslims": "باكستانيون مسلمون",
    "palestinian muslims": "فلسطينيون مسلمون",
    "panamanian muslims": "بنميون مسلمون",
    "persecution of bengali muslims": "اضطهاد بنغاليون مسلمون",
    "persecution of ottoman muslims": "اضطهاد عثمانيون مسلمون",
    "peruvian muslims": "بيرويون مسلمون",
    "polish muslims": "بولنديون مسلمون",
    "portuguese muslims": "برتغاليون مسلمون",
    "puerto rican muslims": "بورتوريكيون مسلمون",
    "qatari muslims": "قطريون مسلمون",
    "romanian muslims": "رومان مسلمون",
    "russian muslims": "روس مسلمون",
    "rwandan muslims": "روانديون مسلمون",
    "salvadoran muslims": "سلفادوريون مسلمون",
    "saudi arabian muslims": "سعوديون مسلمون",
    "scottish muslims": "إسكتلنديون مسلمون",
    "senegalese muslims": "سنغاليون مسلمون",
    "serbian muslims": "صرب مسلمون",
    "sierra leonean muslims": "سيراليونيون مسلمون",
    "singaporean muslims": "سنغافوريون مسلمون",
    "somalian muslims": "صوماليون مسلمون",
    "south american muslims": "أمريكيون جنوبيون مسلمون",
    "south korean muslims": "كوريون جنوبيون مسلمون",
    "south sudanese muslims": "سودانيون جنوبيون مسلمون",
    "soviet muslims": "سوفيت مسلمون",
    "spanish muslims": "إسبان مسلمون",
    "sri lankan muslims": "سريلانكيون مسلمون",
    "sudanese muslims": "سودانيون مسلمون",
    "surinamese muslims": "سوريناميون مسلمون",
    "swedish muslims": "سويديون مسلمون",
    "swiss muslims": "سويسريون مسلمون",
    "syrian muslims": "سوريون مسلمون",
    "taiwanese muslims": "تايوانيون مسلمون",
    "tajikistani muslims": "طاجيك مسلمون",
    "tamil muslims": "تاميليون مسلمون",
    "tanzanian muslims": "تنزانيون مسلمون",
    "thai muslims": "تايلنديون مسلمون",
    "tongan muslims": "تونغانيون مسلمون",
    "trinidad and tobago muslims": "ترنيداديون مسلمون",
    "tunisian muslims": "تونسيون مسلمون",
    "turkish cypriot muslims": "قبرصيون شماليون مسلمون",
    "turkish muslims": "أتراك مسلمون",
    "turkmenistan muslims": "تركمانيون مسلمون",
    "ugandan muslims": "أوغنديون مسلمون",
    "ukrainian muslims": "أوكرانيون مسلمون",
    "uruguayan muslims": "أوروغويانيون مسلمون",
    "uzbekistani muslims": "أوزبكستانيون مسلمون",
    "venezuelan muslims": "فنزويليون مسلمون",
    "vietnamese muslims": "فيتناميون مسلمون",
    "welsh muslims": "ويلزيون مسلمون",
    "yemeni muslims": "يمنيون مسلمون",
    "yugoslav muslims": "يوغسلافيون مسلمون",
    "zambian muslims": "زامبيون مسلمون",
    "zimbabwean muslims": "زيمبابويون مسلمون",
    "anti-muslim sentiment": "مشاعر معادية للمسلمون",
}

data2 = {
    "muslim league (2050–2050)": "الدوري المسلم (2050–2050)",
    "timelines of muslim history": "التسلسل الزمني تاريخ مسلم",
    "muslim families": "عائلات مسلمة",
    "muslim festivals": "مهرجانات مسلمة",
    "muslim league (1947–1958)": "الدوري المسلم (1947–1958)",
    "muslim members of united states house of representatives": "مسلمون أعضاء في مجلس النواب الأمريكي",
    "muslim nationalism in south asia": "قومية مسلمة في جنوب آسيا",
    "bengali muslim surnames": "ألقاب مسلمة بنغالية",
    "muslim rebellions": "تمردات مسلمة",
}


@pytest.mark.parametrize("category, expected", religions_data.items(), ids=religions_data.keys())
def test_muslims(category: str, expected: str) -> None:
    assert resolve_label_ar(category) == expected


to_test = [
    ("test_muslims_1", religions_data),
    # ("test_muslims_2", data2),
]

test_dump_all = make_dump_test_name_data(to_test, resolve_label_ar, run_same=False)



"""
Core data models for translation formatting.
This package contains the base classes and structures used for representing
and processing different translation patterns.
"""

from .model_data import FormatData

# from .model_data_base import FormatDataBase
from .model_data_form import FormatDataFrom
from .model_data_time import YearFormatData
from .model_data_v2 import FormatDataV2

__all__ = [
    "FormatDataFrom",
    "FormatDataV2",
    "YearFormatData",
    "FormatData",
    # "FormatDataBase",
]



"""
Tests
"""

import pytest

from ArWikiCats.new_resolvers.films_resolvers import main_films_resolvers
from utils.dump_runner import make_dump_test_name_data_callback

test_data3 = {
    "animated short film comics": "قصص مصورة رسوم متحركة قصيرة",
    "animated short film film series": "سلاسل أفلام رسوم متحركة قصيرة",
    "animated short film soap opera": "مسلسلات طويلة رسوم متحركة قصيرة",
    "animated short film television episodes": "حلقات تلفزيونية رسوم متحركة قصيرة",
    "animated short film television films": "أفلام تلفزيونية رسوم متحركة قصيرة",
    "animated short film television news": "أخبار تلفزيونية رسوم متحركة قصيرة",
    "animated short film television programmes": "برامج تلفزيونية رسوم متحركة قصيرة",
    "animated short film television programs": "برامج تلفزيونية رسوم متحركة قصيرة",
    "animated short film television series": "مسلسلات تلفزيونية رسوم متحركة قصيرة",
    "animated short film video games": "ألعاب فيديو رسوم متحركة قصيرة",
    "animated short film web series": "مسلسلات ويب رسوم متحركة قصيرة",
    "comic science fiction comics": "قصص مصورة خيالية علمية كوميدية",
    "comic science fiction film series": "سلاسل أفلام خيالية علمية كوميدية",
    "comic science fiction soap opera": "مسلسلات طويلة خيالية علمية كوميدية",
    "comic science fiction television episodes": "حلقات تلفزيونية خيالية علمية كوميدية",
    "comic science fiction television films": "أفلام تلفزيونية خيالية علمية كوميدية",
    "comic science fiction television news": "أخبار تلفزيونية خيالية علمية كوميدية",
    "comic science fiction television programmes": "برامج تلفزيونية خيالية علمية كوميدية",
    "comic science fiction television programs": "برامج تلفزيونية خيالية علمية كوميدية",
    "comic science fiction television series": "مسلسلات تلفزيونية خيالية علمية كوميدية",
    "comic science fiction video games": "ألعاب فيديو خيالية علمية كوميدية",
    "comic science fiction web series": "مسلسلات ويب خيالية علمية كوميدية",
    "television comics": "قصص مصورة تلفزيونية",
    "television episodes": "حلقات تلفزيونية",
    "television film series": "سلاسل أفلام تلفزيونية",
    "television films": "أفلام تلفزيونية",
    "television miniseries": "مسلسلات قصيرة تلفزيونية",
    "television news": "أخبار تلفزيونية",
    "television programmes": "برامج تلفزيونية",
    "television programs": "برامج تلفزيونية",
    "television series": "مسلسلات تلفزيونية",
    "television soap opera": "مسلسلات طويلة تلفزيونية",
    "television video games": "ألعاب فيديو تلفزيونية",
    "television web series": "مسلسلات ويب تلفزيونية",
    "comics": "قصص مصورة",
    "film series": "سلاسل أفلام",
    "soap opera": "مسلسلات طويلة",
    "video games": "ألعاب فيديو",
    "web series": "مسلسلات ويب",
}


@pytest.mark.parametrize("category, expected", test_data3.items(), ids=test_data3.keys())
@pytest.mark.fast
def test_get_films_key_tyty_new(category: str, expected: str) -> None:
    label = main_films_resolvers(category)
    assert label == expected


to_test = [
    ("test_Films_tyty", test_data3, main_films_resolvers),
]

test_dump_all = make_dump_test_name_data_callback(to_test, run_same=True)



"""
Package for resolving job titles and occupations in category names.
This package provides specialized resolvers for male and female job titles,
as well as religious occupations.
"""

import functools
import logging

from . import mens, relegin_jobs_new, womens

logger = logging.getLogger(__name__)


@functools.lru_cache(maxsize=10000)
def main_jobs_resolvers(normalized_category) -> str:
    """
    Resolve a job category name to a standardized jobs label.

    Parameters:
        normalized_category (str): Category name to resolve. Leading "category:" prefix, surrounding whitespace, and letter case are ignored.

    Returns:
        str: The resolved jobs category label, or an empty string if no resolver matched.
    """
    normalized_category = normalized_category.strip().lower().replace("category:", "")
    logger.debug("--" * 20)
    logger.debug(f"<><><><><><> <<green>> {normalized_category=}")

    resolved_label = (
        mens.mens_resolver_labels(normalized_category)
        or womens.womens_resolver_labels(normalized_category)
        or relegin_jobs_new.new_religions_jobs_with_suffix(normalized_category)
        or ""
    )

    logger.info(f"<<yellow>> end {normalized_category=}, {resolved_label=}")
    return resolved_label


__all__ = [
    "main_jobs_resolvers",
]



"""
Tests
"""

import pytest

from ArWikiCats.new_resolvers.sports_resolvers import main_sports_resolvers

data_0 = {}

data_1 = {
    "badminton world cup": "كأس العالم لتنس الريشة",
    "biathlon world cup": "كأس العالم للبياثلون",
    "boxing world cup": "كأس العالم للبوكسينغ",
    "men's hockey world cup": "كأس العالم للهوكي للرجال",
    "national rugby union premier leagues": "دوريات اتحاد رجبي وطنية من الدرجة الممتازة",
    "netball world cup": "كأس العالم لكرة الشبكة",
    "rugby league world cup": "كأس العالم لدوري الرجبي",
    "women's cricket world cup": "كأس العالم للكريكت للسيدات",
    "women's hockey world cup": "كأس العالم للهوكي للسيدات",
    "women's softball world cup": "كأس العالم للكرة اللينة للسيدات",
    "wrestling world cup": "كأس العالم للمصارعة",
}


@pytest.mark.parametrize("category, expected_key", data_1.items(), ids=data_1.keys())
@pytest.mark.fast
def test_new_data(category: str, expected_key: str) -> None:
    label = main_sports_resolvers(category)
    assert label == expected_key



#
import pytest

from ArWikiCats import resolve_label_ar
from utils.dump_runner import make_dump_test_name_data

non_fiction_data_by = {
    "Turkish non-fiction writers by century": "كتاب غير روائيين أتراك حسب القرن",
    "Australian non-fiction writers by century": "كتاب غير روائيين أستراليون حسب القرن",
    "Brazilian non-fiction writers by century": "كتاب غير روائيين برازيليون حسب القرن",
    "Portuguese non-fiction writers by century": "كتاب غير روائيين برتغاليون حسب القرن",
    "Puerto Rican non-fiction writers by century": "كتاب غير روائيين بورتوريكيون حسب القرن",
    "Czech non-fiction writers by century": "كتاب غير روائيين تشيكيون حسب القرن",
    "Welsh non-fiction writers by century": "كتاب غير روائيين ويلزيون حسب القرن",
    "Jewish non-fiction writers by nationality": "كتاب غير روائيين يهود حسب الجنسية",
    "Greek non-fiction writers by century": "كتاب غير روائيين يونانيون حسب القرن",
    "Mexican non-fiction writers by century": "كتاب غير روائيين مكسيكيون حسب القرن",
    "Non-fiction writers from Northern Ireland by century": "كتاب غير روائيين من أيرلندا الشمالية حسب القرن",
    "Non-fiction writers by ethnicity": "كتاب غير روائيين حسب المجموعة العرقية",
    "17th-century non-fiction writers by nationality": "كتاب غير روائيين في القرن 17 حسب الجنسية",
    "18th-century non-fiction writers by nationality": "كتاب غير روائيين في القرن 18 حسب الجنسية",
}

non_fiction_data_from = {
    "20th-century non-fiction writers from Northern Ireland": "كتاب غير روائيين من أيرلندا الشمالية في القرن 20",
    "18th-century non-fiction writers from the Russian Empire": "كتاب غير روائيين من الإمبراطورية الروسية في القرن 18",
}

non_fiction_data_nats = {
    "Salvadoran non-fiction writers": "كتاب غير روائيين سلفادوريون",
    "Non-fiction writers about the United States": "كتاب غير روائيين عن الولايات المتحدة",
    "Non-fiction writers about organized crime": "كتاب غير روائيين عن جريمة منظمة",
    "Non-fiction writers about California": "كتاب غير روائيين عن كاليفورنيا",
    "Croatian non-fiction writers": "كتاب غير روائيين كروات",
    "Moldovan non-fiction writers": "كتاب غير روائيين مولدوفيون",
    "Nepalese non-fiction writers": "كتاب غير روائيين نيباليون",
    "Nicaraguan non-fiction writers": "كتاب غير روائيين نيكاراغويون",
    "British non-fiction environmental writers": "كتاب بيئة غير روائيين بريطانيون",
    "Jordanian non-fiction writers": "كتاب غير روائيين أردنيون",
    "Bahraini non-fiction writers": "كتاب غير روائيين بحرينيون",
    "Bulgarian non-fiction writers": "كتاب غير روائيين بلغاريون",
    "Panamanian non-fiction writers": "كتاب غير روائيين بنميون",
    "Burundian non-fiction writers": "كتاب غير روائيين بورونديون",
    "Gibraltarian non-fiction writers": "كتاب غير روائيين جبل طارقيون",
    "Algerian non-fiction writers": "كتاب غير روائيين جزائريون",
}

non_fiction_data_male = {
    # "New Zealand male non-fiction writers": "كتاب غير روائيين ذكور نيوزيلنديون",
    "Turkish male non-fiction writers": "كتاب غير روائيين ذكور أتراك",
    "Argentine male non-fiction writers": "كتاب غير روائيين ذكور أرجنتينيون",
    "Albanian male non-fiction writers": "كتاب غير روائيين ذكور ألبان",
    "Estonian male non-fiction writers": "كتاب غير روائيين ذكور إستونيون",
    "Israeli male non-fiction writers": "كتاب غير روائيين ذكور إسرائيليون",
    "Scottish male non-fiction writers": "كتاب غير روائيين ذكور إسكتلنديون",
    "Pakistani male non-fiction writers": "كتاب غير روائيين ذكور باكستانيون",
    "Brazilian male non-fiction writers": "كتاب غير روائيين ذكور برازيليون",
    "Portuguese male non-fiction writers": "كتاب غير روائيين ذكور برتغاليون",
    "Puerto Rican male non-fiction writers": "كتاب غير روائيين ذكور بورتوريكيون",
    "Bolivian male non-fiction writers": "كتاب غير روائيين ذكور بوليفيون",
    "Peruvian male non-fiction writers": "كتاب غير روائيين ذكور بيرويون",
    "Trinidad and Tobago male non-fiction writers": "كتاب غير روائيين ذكور ترنيداديون",
    "Czech male non-fiction writers": "كتاب غير روائيين ذكور تشيكيون",
    "Chilean male non-fiction writers": "كتاب غير روائيين ذكور تشيليون",
    "Jamaican male non-fiction writers": "كتاب غير روائيين ذكور جامايكيون",
    "Russian male non-fiction writers": "كتاب غير روائيين ذكور روس",
    "Romanian male non-fiction writers": "كتاب غير روائيين ذكور رومان",
    "Soviet male non-fiction writers": "كتاب غير روائيين ذكور سوفيت",
    "Swiss male non-fiction writers": "كتاب غير روائيين ذكور سويسريون",
    "Serbian male non-fiction writers": "كتاب غير روائيين ذكور صرب",
    "Chinese male non-fiction writers": "كتاب غير روائيين ذكور صينيون",
    "Palestinian male non-fiction writers": "كتاب غير روائيين ذكور فلسطينيون",
    "Venezuelan male non-fiction writers": "كتاب غير روائيين ذكور فنزويليون",
    "Finnish male non-fiction writers": "كتاب غير روائيين ذكور فنلنديون",
    "Cuban male non-fiction writers": "كتاب غير روائيين ذكور كوبيون",
    "Colombian male non-fiction writers": "كتاب غير روائيين ذكور كولومبيون",
    "Luxembourgian male non-fiction writers": "كتاب غير روائيين ذكور لوكسمبورغيون",
    "Lithuanian male non-fiction writers": "كتاب غير روائيين ذكور ليتوانيون",
    "Hungarian male non-fiction writers": "كتاب غير روائيين ذكور مجريون",
    "Egyptian male non-fiction writers": "كتاب غير روائيين ذكور مصريون",
    "Mexican male non-fiction writers": "كتاب غير روائيين ذكور مكسيكيون",
    "Moldovan male non-fiction writers": "كتاب غير روائيين ذكور مولدوفيون",
    "Norwegian male non-fiction writers": "كتاب غير روائيين ذكور نرويجيون",
    "Austrian male non-fiction writers": "كتاب غير روائيين ذكور نمساويون",
    "Haitian male non-fiction writers": "كتاب غير روائيين ذكور هايتيون",
    "Dutch male non-fiction writers": "كتاب غير روائيين ذكور هولنديون",
    "Welsh male non-fiction writers": "كتاب غير روائيين ذكور ويلزيون",
    "Japanese male non-fiction writers": "كتاب غير روائيين ذكور يابانيون",
    "Greek male non-fiction writers": "كتاب غير روائيين ذكور يونانيون",
}

non_fiction_data_nat_with_time = {
    "20th-century Turkish non-fiction writers": "كتاب غير روائيين أتراك في القرن 20",
    "21st-century Turkish non-fiction writers": "كتاب غير روائيين أتراك في القرن 21",
    "19th-century Australian non-fiction writers": "كتاب غير روائيين أستراليون في القرن 19",
    "17th-century Irish non-fiction writers": "كتاب غير روائيين أيرلنديون في القرن 17",
    "18th-century Irish non-fiction writers": "كتاب غير روائيين أيرلنديون في القرن 18",
    "19th-century Spanish non-fiction writers": "كتاب غير روائيين إسبان في القرن 19",
    "20th-century Spanish non-fiction writers": "كتاب غير روائيين إسبان في القرن 20",
    "17th-century Scottish non-fiction writers": "كتاب غير روائيين إسكتلنديون في القرن 17",
    "18th-century Scottish non-fiction writers": "كتاب غير روائيين إسكتلنديون في القرن 18",
    "19th-century Scottish non-fiction writers": "كتاب غير روائيين إسكتلنديون في القرن 19",
    "20th-century Scottish non-fiction writers": "كتاب غير روائيين إسكتلنديون في القرن 20",
    "19th-century Italian non-fiction writers": "كتاب غير روائيين إيطاليون في القرن 19",
    "19th-century Brazilian non-fiction writers": "كتاب غير روائيين برازيليون في القرن 19",
    "20th-century Brazilian non-fiction writers": "كتاب غير روائيين برازيليون في القرن 20",
    "21st-century Brazilian non-fiction writers": "كتاب غير روائيين برازيليون في القرن 21",
    "20th-century Portuguese non-fiction writers": "كتاب غير روائيين برتغاليون في القرن 20",
    "21st-century Portuguese non-fiction writers": "كتاب غير روائيين برتغاليون في القرن 21",
    "19th-century Belgian non-fiction writers": "كتاب غير روائيين بلجيكيون في القرن 19",
    "20th-century Bangladeshi non-fiction writers": "كتاب غير روائيين بنغلاديشيون في القرن 20",
    "21st-century Bangladeshi non-fiction writers": "كتاب غير روائيين بنغلاديشيون في القرن 21",
    "19th-century Puerto Rican non-fiction writers": "كتاب غير روائيين بورتوريكيون في القرن 19",
    "20th-century Puerto Rican non-fiction writers": "كتاب غير روائيين بورتوريكيون في القرن 20",
    "21st-century Puerto Rican non-fiction writers": "كتاب غير روائيين بورتوريكيون في القرن 21",
    "19th-century Czech non-fiction writers": "كتاب غير روائيين تشيكيون في القرن 19",
    "20th-century Czech non-fiction writers": "كتاب غير روائيين تشيكيون في القرن 20",
    "21st-century Czech non-fiction writers": "كتاب غير روائيين تشيكيون في القرن 21",
    "18th-century Danish non-fiction writers": "كتاب غير روائيين دنماركيون في القرن 18",
    "19th-century Danish non-fiction writers": "كتاب غير روائيين دنماركيون في القرن 19",
    "19th-century Swedish non-fiction writers": "كتاب غير روائيين سويديون في القرن 19",
    "21st-century Swedish non-fiction writers": "كتاب غير روائيين سويديون في القرن 21",
    "20th-century Chinese non-fiction writers": "كتاب غير روائيين صينيون في القرن 20",
    "21st-century Chinese non-fiction writers": "كتاب غير روائيين صينيون في القرن 21",
    "20th-century Finnish non-fiction writers": "كتاب غير روائيين فنلنديون في القرن 20",
    "21st-century South Korean non-fiction writers": "كتاب غير روائيين كوريون جنوبيون في القرن 21",
    "21st-century Lithuanian non-fiction writers": "كتاب غير روائيين ليتوانيون في القرن 21",
    "20th-century Egyptian non-fiction writers": "كتاب غير روائيين مصريون في القرن 20",
    "21st-century Egyptian non-fiction writers": "كتاب غير روائيين مصريون في القرن 21",
    "19th-century Mexican non-fiction writers": "كتاب غير روائيين مكسيكيون في القرن 19",
    "20th-century Mexican non-fiction writers": "كتاب غير روائيين مكسيكيون في القرن 20",
    "21st-century Mexican non-fiction writers": "كتاب غير روائيين مكسيكيون في القرن 21",
    "21st-century Norwegian non-fiction writers": "كتاب غير روائيين نرويجيون في القرن 21",
    "20th-century Austrian non-fiction writers": "كتاب غير روائيين نمساويون في القرن 20",
    "21st-century Austrian non-fiction writers": "كتاب غير روائيين نمساويون في القرن 21",
    "17th-century Dutch non-fiction writers": "كتاب غير روائيين هولنديون في القرن 17",
    "18th-century Dutch non-fiction writers": "كتاب غير روائيين هولنديون في القرن 18",
    "19th-century Dutch non-fiction writers": "كتاب غير روائيين هولنديون في القرن 19",
    "20th-century Welsh non-fiction writers": "كتاب غير روائيين ويلزيون في القرن 20",
    "21st-century Welsh non-fiction writers": "كتاب غير روائيين ويلزيون في القرن 21",
    "20th-century Japanese non-fiction writers": "كتاب غير روائيين يابانيون في القرن 20",
    "21st-century Japanese non-fiction writers": "كتاب غير روائيين يابانيون في القرن 21",
    "19th-century Greek non-fiction writers": "كتاب غير روائيين يونانيون في القرن 19",
    "20th-century Greek non-fiction writers": "كتاب غير روائيين يونانيون في القرن 20",
    "21st-century Greek non-fiction writers": "كتاب غير روائيين يونانيون في القرن 21",
}

to_test = [
    ("test_non_fiction_data_by", non_fiction_data_by),
    ("test_non_fiction_data_from", non_fiction_data_from),
    ("test_non_fiction_data_nats", non_fiction_data_nats),
    ("test_non_fiction_data_male", non_fiction_data_male),
    ("test_non_fiction_data_nat_with_time", non_fiction_data_nat_with_time),
]

non_fiction_data_all = {
    **non_fiction_data_by,
    **non_fiction_data_from,
    **non_fiction_data_nats,
    **non_fiction_data_male,
    **non_fiction_data_nat_with_time,
}


@pytest.mark.parametrize("category, expected", non_fiction_data_all.items(), ids=non_fiction_data_all.keys())
@pytest.mark.slow
def test_non_fiction_writers(category: str, expected: str) -> None:
    label = resolve_label_ar(category)
    assert label == expected


test_dump_all = make_dump_test_name_data(to_test, resolve_label_ar, run_same=True)



"""University labelling helpers."""

from __future__ import annotations

import functools
import logging
from typing import Dict

from ..translations import CITY_TRANSLATIONS_LOWER
from ..translations_formats import FormatData

logger = logging.getLogger(__name__)

CITY_LOWER = {
    "chandler, oklahoma": "تشاندلر (أوكلاهوما)",
    "changchun": "تشانغتشون",
    "changde": "تشانغده",
    "changhua county": "مقاطعة تشانغوا",
    "changning, hunan": "تشانغ نينغ، هونان",
    "changnyeong county": "محافظة تشانغنيونغ",
    "changsha": "تشانغشا",
    "changzhi": "تشانغ تشى",
    "changzhou": "تشانغتشو",
    "chanhassen, minnesota": "تشانهاسين (منيسوتا)",
    "chania": "خانية",
    "channahon, illinois": "تشاناهون (إلينوي)",
    "chaohu": "شاوهو",
    "chaoyang, liaoning": "تشاويانغ",
    "chaozhou": "شاوزو",
    "chapayevsk": "تشاباييفسك",
    "chapin, south carolina": "تشابين (كارولاينا الجنوبية)",
    "chaplin, connecticut": "تشابين (كونيتيكت)",
    "chapmanville, west virginia": "تشامبانفيل (فرجينيا الغربية)",
    "chardon, ohio": "تشاردن",
    "port townsend, washington": "بورت تاونسند",
    "portage": "بورتج",
    "portage la prairie": "بورتاج لابريري",
    "portage, indiana": "بورتاغ",
    "portage, wisconsin": "بورتاغ (ويسكونسن)",
    "portalegre, portugal": "بورتاليغري (البرتغال)",
    "portales, new mexico": "بورتاليس",
    "porter": "بورتر",
    "porterville, california": "بورتيرفيل (كاليفورنيا)",
    "portland, maine": "بورتلاند (مين)",
    "portland, oregon": "بورتلاند (أوريغن)",
    "porto": "بورتو",
    "porto alegre": "بورتو أليغري",
    "porto-novo": "بورتو نوفو",
    "portola valley, california": "بورتولا فالي (كاليفورنيا)",
    "portorož": "بورتوروز",
    "portsmouth, new hampshire": "بورتسموث (نيوهامشير)",
    "portsmouth, ohio": "بورتسموث (أوهايو)",
    "portsmouth, rhode island": "بورتسموث (رود آيلاند)",
    "portsmouth, virginia": "بورتسموث (فرجينيا)",
    "portuguese malacca": "ملقا البرتغالية",
    "porvoo": "بورفو",
    "posadas, misiones": "بوساداس (ميسيونيس)",
    "posey": "بوسي",
    "potenza": "بوتنسا",
}

CITY_LOWER.update(CITY_TRANSLATIONS_LOWER)

MAJORS: Dict[str, str] = {
    "medical sciences": "للعلوم الطبية",
    "international university": "الدولية",
    "art": "للفنون",
    "arts": "للفنون",
    "biology": "للبيولوجيا",
    "chemistry": "للشيمية",
    "computer science": "للكمبيوتر",
    "economics": "للاقتصاد",
    "education": "للتعليم",
    "engineering": "للهندسة",
    "geography": "للجغرافيا",
    "geology": "للجيولوجيا",
    "history": "للتاريخ",
    "law": "للقانون",
    "mathematics": "للرياضيات",
    "technology": "للتكنولوجيا",
    "physics": "للفيزياء",
    "psychology": "للصحة",
    "sociology": "للأمن والسلوك",
    "political science": "للسياسة",
    "social science": "للأمن والسلوك",
    "social sciences": "للأمن والسلوك",
    "science and technology": "للعلوم والتكنولوجيا",
    "science": "للعلوم",
    "reading": "للقراءة",
    "applied sciences": "للعلوم التطبيقية",
}

UNIVERSITIES_TABLES: Dict[str, str] = {
    "national maritime university": "جامعة {} الوطنية البحرية",
    "national university": "جامعة {} الوطنية",
}

for major, arabic_label in MAJORS.items():
    normalized_major = major.lower()
    template = f"جامعة {{}} {arabic_label}"
    UNIVERSITIES_TABLES[f"university of {normalized_major}"] = template
    UNIVERSITIES_TABLES[f"university-of-{normalized_major}"] = template
    UNIVERSITIES_TABLES[f"university of the {normalized_major}"] = template
    UNIVERSITIES_TABLES[f"university-of-the-{normalized_major}"] = template

# Build formatted_data for FormatData bot
_formatted_university_data = {}
for key, template in UNIVERSITIES_TABLES.items():
    ar_template = template.replace("{}", "{city_ar}")
    # Patterns for "{city} {university_key}"
    _formatted_university_data[f"{{city}} {key}"] = ar_template
    # Patterns for "{university_key}, {city}"
    _formatted_university_data[f"{key}, {{city}}"] = ar_template
    # Patterns for "{university_key} {city}"
    _formatted_university_data[f"{key} {{city}}"] = ar_template

_university_bot = FormatData(
    formatted_data=_formatted_university_data,
    data_list=CITY_LOWER,
    key_placeholder="{city}",
    value_placeholder="{city_ar}",
)


def _normalise_category(category: str) -> str:
    """Lowercase and strip ``category`` while removing ``Category:`` prefix."""

    normalized = category.lower().strip()
    if normalized.startswith("category:"):
        normalized = normalized[len("category:") :].strip()
    return normalized


@functools.lru_cache(maxsize=2048)
def resolve_university_category(category: str) -> str:
    """
    Resolve a normalized university-related category into its Arabic university label.

    Returns:
        str: The Arabic university label formatted with the resolved city name, or an empty string if no mapping is found.
    """

    normalized_category = _normalise_category(category)

    logger.info(f"<<lightblue>>>> vvvvvvvvvvvv start, (category:{normalized_category}) vvvvvvvvvvvv ")

    university_label = _university_bot.search(normalized_category)

    if university_label:
        logger.info(f"<<lightblue>>>>>>: new {university_label=} ")

    logger.info("<<lightblue>>>> ^^^^^^^^^ end ^^^^^^^^^ ")
    return university_label


__all__ = [
    "resolve_university_category",
]



import pytest

from ArWikiCats import resolve_label_ar
from utils.dump_runner import make_dump_test_name_data

data1 = {
    "People in arts occupations by nationality": "أشخاص في مهن فنية حسب الجنسية",
    "People of Ivorian descent": "أشخاص من أصل إيفواري",
    "Polish women by occupation": "بولنديات حسب المهنة",
    "Portuguese healthcare managers": "مدراء رعاية صحية برتغاليون",
    "Prisoners and detainees of Afghanistan": "سجناء ومعتقلون في أفغانستان",
    "Prisons in Afghanistan": "سجون في أفغانستان",
    "Scholars by subfield": "دارسون حسب الحقل الفرعي",
    "Women in business by nationality": "سيدات أعمال حسب الجنسية",
    "women in business": "سيدات أعمال",
}

data2 = {
    "Iranian nuclear medicine physicians": "أطباء طب نووي إيرانيون",
    "Israeli people of Northern Ireland descent": "إسرائيليون من أصل أيرلندي شمالي",
    "Ivorian diaspora in Asia": "شتات إيفواري في آسيا",
    "Medical doctors by specialty and nationality": "أطباء حسب التخصص والجنسية",
    "Multi-instrumentalists": "عازفون على عدة آلات",
    "People by nationality and status": "أشخاص حسب الجنسية والحالة",
}

data3 = {
    "Canadian nuclear medicine physicians": "أطباء طب نووي كنديون",
    "Croatian nuclear medicine physicians": "أطباء طب نووي كروات",
    "Expatriate male actors in New Zealand": "ممثلون ذكور مغتربون في نيوزيلندا",
    "Expatriate male actors": "ممثلون ذكور مغتربون",
    "German nuclear medicine physicians": "أطباء طب نووي ألمان",
    "Immigrants to New Zealand": "مهاجرون إلى نيوزيلندا",
    "Immigration to New Zealand": "الهجرة إلى نيوزيلندا",
    "Internees at the Sheberghan Prison": "معتقلون في سجن شيبرغان",
}


data4 = {
    "Afghan diplomats": "دبلوماسيون أفغان",
    "Ambassadors of Afghanistan": "سفراء أفغانستان",
    "Ambassadors of the Ottoman Empire": "سفراء الدولة العثمانية",
    "Ambassadors to the Ottoman Empire": "سفراء لدى الدولة العثمانية",
    "American nuclear medicine physicians": "أطباء طب نووي أمريكيون",
    "Argentine multi-instrumentalists": "عازفون على عدة آلات أرجنتينيون",
    "Attacks on diplomatic missions": "هجمات على بعثات دبلوماسية",
    "Australian Internet celebrities": "مشاهير إنترنت أستراليون",
}

to_test = [
    ("test_people_1", data1),
    ("test_people_2", data2),
    ("test_people_3", data3),
    ("test_people_4", data4),
]


@pytest.mark.parametrize("category, expected", data1.items(), ids=data1.keys())
@pytest.mark.fast
def test_people_1(category: str, expected: str) -> None:
    label = resolve_label_ar(category)
    assert label == expected


@pytest.mark.parametrize("category, expected", data2.items(), ids=data2.keys())
@pytest.mark.fast
def test_people_2(category: str, expected: str) -> None:
    label = resolve_label_ar(category)
    assert label == expected


@pytest.mark.parametrize("category, expected", data3.items(), ids=data3.keys())
@pytest.mark.fast
def test_people_3(category: str, expected: str) -> None:
    label = resolve_label_ar(category)
    assert label == expected


@pytest.mark.parametrize("category, expected", data4.items(), ids=data4.keys())
@pytest.mark.fast
def test_people_4(category: str, expected: str) -> None:
    label = resolve_label_ar(category)
    assert label == expected


test_dump_all = make_dump_test_name_data(to_test, resolve_label_ar, run_same=False)



"""Tests for build_sport_keys.py sport key translation utilities."""

from ArWikiCats.translations.data_builders.build_sport_keys import (
    SportKeyRecord,
    SportKeyTables,
    _apply_aliases,
    _build_tables,
    _coerce_record,
    _copy_record,
    _generate_variants,
    _initialise_tables,
    _load_base_records,
)


class TestCoerceRecord:
    """Tests for _coerce_record function."""

    def test_coerce_full_record(self) -> None:
        raw = {"label": "كرة القدم", "jobs": "رياضة", "team": "للرياضة", "olympic": "أولمبية"}
        result = _coerce_record(raw)

        assert result["label"] == "كرة القدم"
        assert result["jobs"] == "رياضة"
        assert result["team"] == "للرياضة"
        assert result["olympic"] == "أولمبية"

    def test_coerce_partial_record(self) -> None:
        raw = {"label": "كرة السلة"}
        result = _coerce_record(raw)

        assert result["label"] == "كرة السلة"
        assert result["jobs"] == ""
        assert result["team"] == ""
        assert result["olympic"] == ""

    def test_coerce_empty_record(self) -> None:
        result = _coerce_record({})
        assert result == {"label": "", "jobs": "", "team": "", "olympic": ""}

    def test_coerce_with_numeric_values(self) -> None:
        raw = {"label": 123, "jobs": None}
        result = _coerce_record(raw)

        assert result["label"] == "123"
        assert result["jobs"] == "None"


class TestLoadBaseRecords:
    """Tests for _load_base_records function."""

    def test_load_valid_mapping(self) -> None:
        data = {
            "football": {"label": "كرة القدم", "jobs": "رياضة"},
            "basketball": {"label": "كرة السلة"},
        }
        result = _load_base_records(data)

        assert "football" in result
        assert result["football"]["label"] == "كرة القدم"
        assert result["basketball"]["label"] == "كرة السلة"

    def test_load_ignores_entries_with_ignore_key(self) -> None:
        data = {
            "football": {"label": "كرة القدم"},
            "ignored": {"label": "تجاهل", "ignore": True},
        }
        result = _load_base_records(data)

        assert "football" in result
        assert "ignored" not in result

    def test_load_non_mapping_returns_empty(self) -> None:
        result = _load_base_records("not a mapping")
        assert result == {}

    def test_load_skips_malformed_entries(self) -> None:
        data = {
            "valid": {"label": "صحيح"},
            "invalid": "not a mapping",
        }
        result = _load_base_records(data)

        assert "valid" in result
        assert result["valid"]["label"] == "صحيح"


class TestCopyRecord:
    """Tests for _copy_record function."""

    def test_copy_without_overrides(self) -> None:
        record: SportKeyRecord = {
            "label": "كرة القدم",
            "jobs": "رياضية",
            "team": "للرياضة",
            "olympic": "أولمبية",
        }
        result = _copy_record(record)

        assert result == record

    def test_copy_with_overrides(self) -> None:
        record: SportKeyRecord = {"label": "كرة القدم", "jobs": "", "team": "", "olympic": ""}
        result = _copy_record(record, label="كرة السلة", jobs="كرة سلة")

        assert result["label"] == "كرة السلة"
        assert result["jobs"] == "كرة سلة"
        assert result["team"] == ""
        assert result["olympic"] == ""

    def test_copy_skips_empty_overrides(self) -> None:
        record: SportKeyRecord = {"label": "كرة القدم", "jobs": "", "team": "", "olympic": ""}
        result = _copy_record(record, label="", jobs="new jobs")

        assert result["label"] == "كرة القدم"  # unchanged
        assert result["jobs"] == "new jobs"


class TestApplyAliases:
    """Tests for _apply_aliases function."""

    def test_apply_aliases_adds_entries(self) -> None:
        records: dict[str, SportKeyRecord] = {"football": {"label": "كرة القدم", "jobs": "", "team": "", "olympic": ""}}
        aliases = {"soccer": "football"}

        _apply_aliases(records, aliases)

        assert "soccer" in records
        assert records["soccer"]["label"] == "كرة القدم"

    def test_apply_aliases_missing_source(self) -> None:
        records: dict[str, SportKeyRecord] = {}
        aliases = {"alias": "missing"}

        _apply_aliases(records, aliases)

        assert "alias" not in records


class TestGenerateVariants:
    """Tests for _generate_variants function."""

    def test_generate_racing_variants(self) -> None:
        records = {"athletics": {"label": "ألعاب قوى", "jobs": "ألعاب قوى", "team": "للألعاب القوى", "olympic": ""}}
        result = _generate_variants(records)

        assert "athletics racing" in result
        assert result["athletics racing"]["label"] == "سباق ألعاب قوى"

    def test_skips_existing_racing_sports(self) -> None:
        records = {
            "racing": {"label": "سباق", "jobs": "", "team": "", "olympic": ""},
            "other": {"label": "أخرى", "jobs": "", "team": "", "olympic": ""},
        }
        result = _generate_variants(records)

        assert "racing racing" not in result
        assert "other racing" in result

    def test_generate_wheelchair_variants(self) -> None:
        records = {
            "basketball": {
                "label": "كرة السلة",
                "jobs": "كرة سلة",
                "team": "لكرة السلة",
                "olympic": "",
            }
        }
        result = _generate_variants(records)

        assert "wheelchair basketball" in result
        assert "على الكراسي المتحركة" in result["wheelchair basketball"]["label"]

    def test_wheelchair_only_for_allowed_sports(self) -> None:
        records = {
            "basketball": {"label": "كرة السلة", "jobs": "", "team": "", "olympic": ""},
            "swimming": {"label": "السباحة", "jobs": "", "team": "", "olympic": ""},
        }
        result = _generate_variants(records)

        assert "wheelchair basketball" in result
        assert "wheelchair swimming" not in result


class TestBuildTables:
    """Tests for _build_tables function."""

    def test_build_returns_sport_key_tables(self) -> None:
        records = {
            "football": {
                "label": "كرة القدم",
                "jobs": "كرة قدم",
                "team": "لكرة القدم",
                "olympic": "كرة قدم أولمبية",
            },
            "basketball": {"label": "كرة السلة", "jobs": "", "team": "", "olympic": ""},
        }
        result = _build_tables(records)

        assert isinstance(result, SportKeyTables)
        assert result.label["football"] == "كرة القدم"
        assert result.jobs["football"] == "كرة قدم"
        assert result.team["football"] == "لكرة القدم"
        assert result.olympic["football"] == "كرة قدم أولمبية"
        assert result.label["basketball"] == "كرة السلة"
        assert "basketball" not in result.jobs  # empty values skipped

    def test_build_lowercases_keys(self) -> None:
        records = {"Football": {"label": "كرة القدم", "jobs": "", "team": "", "olympic": ""}}
        result = _build_tables(records)

        assert "football" in result.label
        assert "Football" not in result.label


class TestInitialiseTables:
    """Tests for _initialise_tables function."""

    def test_initialise_loads_and_applies_aliases(self) -> None:
        data = {"football": {"label": "كرة القدم", "jobs": "", "team": "", "olympic": ""}}
        aliases = {"soccer": "football"}

        result = _initialise_tables(data, aliases)

        assert "football" in result
        assert "soccer" in result
        assert result["soccer"]["label"] == "كرة القدم"

    def test_initialise_with_non_mapping_data(self) -> None:
        result = _initialise_tables("invalid", {})
        assert result == {}



"""
Tests
"""

import pytest

from ArWikiCats import resolve_label_ar
from utils.dump_runner import make_dump_test_name_data

test_data_0 = {
    "Slaves from the Mamluk Sultanate": "عبيد من مماليك مصر",
    "Philosophers of technology": "فلاسفة تقنية (تكنولوجيا)",
}

test_data_1 = {
    # "Women's National Basketball Association players from Belgium": "لاعبات الاتحاد الوطني لكرة السلة للسيدات من بلجيكا",
    "1988 war films": "أفلام حربية في 1988",
    "2011 in film by country": "الأفلام في 2011 حسب البلد",
    "20th-century Danish sportsmen": "رياضيون رجال دنماركيون في القرن 20",
    "21st-century Indian murderers": "قتلة هنود في القرن 21",
    "African pan-Africanists": "وحدويون أفارقة أفارقة",
    "American expatriate actresses in India": "ممثلات مغتربات أمريكيات في الهند",
    "American scholars of ancient Greek philosophy": "دارسون أمريكيون في فلسفة يونانية قديمة",
    "Asian Games footballers for Chinese Taipei": "لاعبو كرة قدم في الألعاب الآسيوية في تايبيه الصينية",
    "Australian expatriate actresses in India": "ممثلات مغتربات أستراليات في الهند",
    "Candidates in the 2025 Canadian federal election": "مرشحون في الانتخابات الفيدرالية الكندية 2025",
    "Children's crime films": "أفلام أطفال جريمة",
    "Constituencies of the National Assembly (South Korea)": "دوائر إنتخابية في الجمعية الوطنية (كوريا الجنوبية)",
    "Fantasy adventure novels": "روايات فانتازيا مغامرات",
    "Film characters introduced in 1999": "شخصيات أفلام عرضت في 1999",
    "Judges in Canadian reality television series": "قضاة في مسلسلات تلفزيونية واقعية كندية",
    "Male adult models": "عارضون بالغون ذكور",
    "Members of the Democratic Socialists of America from New York (state)": "أعضاء اشتراكيون ديمقراطيون من أمريكا من ولاية نيويورك",
    "Members of the Democratic Socialists of America from Pennsylvania": "أعضاء اشتراكيون ديمقراطيون من أمريكا من بنسلفانيا",
    "People from the Province of Huesca": "أشخاص من وشقة (مقاطعة)",
    "Poetry of Georgia (country)": "شعر في جورجيا",
    "Polish sportspeople by voivodeship and populated place": "رياضيون بولنديون حسب الفويفود والمكان المأهول",
    "Prisoners sentenced to death by the British military": "مسجونون حكم عليهم بالإعدام بواسطة الجيش البريطاني",
    "Prisoners sentenced to death by the United Kingdom": "مسجونون حكم عليهم بالإعدام بواسطة المملكة المتحدة",
    "Scholars of ancient Greek history": "دارسون من تاريخ يوناني قديم",
    "Scholars of ancient Greek philosophy by nationality": "دارسون من فلسفة يونانية قديمة حسب الجنسية",
    "YouTubers with disabilities": "مشاهير يوتيوب بإعاقات",
    "12th-century deaths from infectious disease": "وفيات بسبب أمراض معدية في القرن 12",
    "14th-century writers from the Crown of Aragon": "كتاب من تاج أرغون في القرن 14",
    "17th-century women from the Republic of Venice": "نساء من جمهورية البندقية في القرن 17",
    "18th-century clergy from the Republic of Geneva": "رجال دين من جمهورية جنيف في القرن 18",
    "2010s supernatural thriller films": "أفلام إثارة خارقة للطبيعة في عقد 2010",
    "2022 Women's Africa Cup of Nations": "كأس الأمم الإفريقية للسيدات 2022",
    "Accidental deaths from falls in the United Kingdom": "وفيات عرضية نتيجة السقوط في المملكة المتحدة",
    "American people executed for robbery": "أمريكيون أعدموا بتهمة السرقة",
    "Environment and climate change ministers of Qatar": "وزراء بيئة وتغير المناخ قطر",
    "Haitian people with disabilities": "هايتيون بإعاقات",
    "Lists of American reality television series episodes": "قوائم حلقات مسلسلات تلفزيونية واقعية أمريكية",
    "Members of the National Council (Switzerland) 1999–2003": "أعضاء المجلس الوطني (سويسرا) 1999–2003",
    "October 1970 in sports": "ألعاب رياضية في أكتوبر 1970",
    "Pan American Games bronze medalists for Canada in athletics (track and field)": "فائزون بميداليات برونزية في دورة الألعاب الأمريكية من كندا في ألعاب قوى المضمار والميدان",
    "Pan American Games gold medalists for Canada in athletics (track and field)": "فائزون بميداليات ذهبية في دورة الألعاب الأمريكية من كندا في ألعاب قوى المضمار والميدان",
    "Pan American Games silver medalists for Canada in athletics (track and field)": "فائزون بميداليات فضية في دورة الألعاب الأمريكية من كندا في ألعاب قوى المضمار والميدان",
    "Relationships of Jeffrey Epstein": "علاقات جيفري إبستين",
    "Secretaries of health of the Philippines": "وزراء صحة فلبينيون",
    "Secretaries of state of France": "وزراء خارجية فرنسيون",
    "Secretaries of the Australian Department of Defence": "وزراء دفاع أستراليون",
    "Somalian people with disabilities": "صوماليون بإعاقات",
    "Supernatural slasher films": "أفلام تقطيع خارقة للطبيعة",
    "United States under secretaries of agriculture": "نواب وزير الزراعة الأمريكي للشؤون المتخصصة",
    "Women's handball players": "لاعبات كرة يد",
    "11th-century people from the Republic of Venice": "أشخاص من جمهورية البندقية في القرن 11",
}


@pytest.mark.parametrize("category,expected", test_data_1.items(), ids=test_data_1.keys())
@pytest.mark.fast
def test_move_by_in(category: str, expected: str) -> None:
    label = resolve_label_ar(category)
    assert label == expected


TEMPORAL_CASES = [
    ("test_data", test_data_1),
]

test_dump_all = make_dump_test_name_data(TEMPORAL_CASES, callback=resolve_label_ar, run_same=True)



#!/usr/bin/python3
"""
Unit tests for model_multi_data_year_from_2.py module.

This module provides tests for MultiDataFormatterYearAndFrom2 class
which combines year-based and "from" relation category translations
with category_relation_mapping support.
"""

import pytest

from ArWikiCats.translations_formats.DataModel import FormatDataFrom
from ArWikiCats.translations_formats.DataModelMulti import MultiDataFormatterBaseHelpers
from ArWikiCats.translations_formats.DataModelMulti.model_multi_data_year_from_2 import (
    MultiDataFormatterYearAndFrom2,
)


class TestMultiDataFormatterYearAndFrom2Init:
    """Tests for MultiDataFormatterYearAndFrom2 initialization."""

    def test_init_basic(self):
        """Test basic initialization of MultiDataFormatterYearAndFrom2."""
        country_bot = FormatDataFrom(
            formatted_data={"{year1} {country1}": "{country1} في {year1}"},
            key_placeholder="{country1}",
            value_placeholder="{country1}",
            search_callback=lambda x: "اليمن" if "yemen" in x.lower() else "",
            match_key_callback=lambda x: "yemen" if "yemen" in x.lower() else "",
        )
        year_bot = FormatDataFrom(
            formatted_data={},
            key_placeholder="{year1}",
            value_placeholder="{year1}",
            search_callback=lambda x: "2020" if "2020" in x else "",
            match_key_callback=lambda x: "2020" if "2020" in x else "",
        )

        bot = MultiDataFormatterYearAndFrom2(country_bot, year_bot)

        assert bot.country_bot is country_bot
        assert bot.other_bot is year_bot
        assert bot.search_first_part is False
        assert bot.data_to_find is None
        assert bot.other_key_first is False
        assert bot.category_relation_mapping == {}

    def test_init_with_category_relation_mapping(self):
        """Test initialization with category_relation_mapping."""
        country_bot = FormatDataFrom(
            formatted_data={},
            key_placeholder="{country1}",
            value_placeholder="{country1}",
            search_callback=lambda x: "",
            match_key_callback=lambda x: "",
        )
        year_bot = FormatDataFrom(
            formatted_data={},
            key_placeholder="{year1}",
            value_placeholder="{year1}",
            search_callback=lambda x: "",
            match_key_callback=lambda x: "",
        )
        mapping = {"from": "من", "in": "في", "by": "بواسطة"}

        bot = MultiDataFormatterYearAndFrom2(country_bot, year_bot, category_relation_mapping=mapping)

        assert "from" in bot.category_relation_mapping
        assert "in" in bot.category_relation_mapping
        assert "by" in bot.category_relation_mapping

    def test_init_with_other_key_first(self):
        """Test initialization with other_key_first=True."""
        country_bot = FormatDataFrom(
            formatted_data={},
            key_placeholder="{country1}",
            value_placeholder="{country1}",
            search_callback=lambda x: "",
            match_key_callback=lambda x: "",
        )
        year_bot = FormatDataFrom(
            formatted_data={},
            key_placeholder="{year1}",
            value_placeholder="{year1}",
            search_callback=lambda x: "",
            match_key_callback=lambda x: "",
        )

        bot = MultiDataFormatterYearAndFrom2(country_bot, year_bot, other_key_first=True)

        assert bot.other_key_first is True

    def test_inherits_from_base_helpers(self):
        """Test that MultiDataFormatterYearAndFrom2 inherits from MultiDataFormatterBaseHelpers."""
        country_bot = FormatDataFrom(
            formatted_data={},
            key_placeholder="{country1}",
            value_placeholder="{country1}",
            search_callback=lambda x: "",
            match_key_callback=lambda x: "",
        )
        year_bot = FormatDataFrom(
            formatted_data={},
            key_placeholder="{year1}",
            value_placeholder="{year1}",
            search_callback=lambda x: "",
            match_key_callback=lambda x: "",
        )

        bot = MultiDataFormatterYearAndFrom2(country_bot, year_bot)

        assert isinstance(bot, MultiDataFormatterBaseHelpers)


class TestMultiDataFormatterYearAndFrom2GetRelationWord:
    """Tests for MultiDataFormatterYearAndFrom2.get_relation_word method."""

    @pytest.fixture
    def bot_with_mapping(self):
        """Create a bot with category_relation_mapping."""
        country_bot = FormatDataFrom(
            formatted_data={},
            key_placeholder="{country1}",
            value_placeholder="{country1}",
            search_callback=lambda x: "",
            match_key_callback=lambda x: "",
        )
        year_bot = FormatDataFrom(
            formatted_data={},
            key_placeholder="{year1}",
            value_placeholder="{year1}",
            search_callback=lambda x: "",
            match_key_callback=lambda x: "",
        )
        mapping = {
            "from": "من",
            "in": "في",
            "published by": "نشرتها",
        }
        return MultiDataFormatterYearAndFrom2(country_bot, year_bot, category_relation_mapping=mapping)

    def test_get_relation_word_from(self, bot_with_mapping):
        """Test get_relation_word with 'from' relation."""
        key, ar = bot_with_mapping.get_relation_word("People from Germany")
        assert key == "from"
        assert ar == "من"

    def test_get_relation_word_in(self, bot_with_mapping):
        """Test get_relation_word with 'in' relation."""
        key, ar = bot_with_mapping.get_relation_word("Buildings in France")
        assert key == "in"
        assert ar == "في"

    def test_get_relation_word_no_match(self, bot_with_mapping):
        """Test get_relation_word when no relation matches."""
        key, ar = bot_with_mapping.get_relation_word("German writers")
        assert key == ""
        assert ar == ""

    def test_get_relation_word_multi_word(self, bot_with_mapping):
        """Test get_relation_word with multi-word relation."""
        key, ar = bot_with_mapping.get_relation_word("Works published by Oxford")
        assert key == "published by"
        assert ar == "نشرتها"


class TestMultiDataFormatterYearAndFrom2ResolveRelationLabel:
    """Tests for MultiDataFormatterYearAndFrom2.resolve_relation_label method."""

    @pytest.fixture
    def bot_with_mapping(self):
        """Create a bot with category_relation_mapping."""
        country_bot = FormatDataFrom(
            formatted_data={},
            key_placeholder="{country1}",
            value_placeholder="{country1}",
            search_callback=lambda x: "",
            match_key_callback=lambda x: "",
        )
        year_bot = FormatDataFrom(
            formatted_data={},
            key_placeholder="{year1}",
            value_placeholder="{year1}",
            search_callback=lambda x: "",
            match_key_callback=lambda x: "",
        )
        mapping = {"from": "من", "in": "في"}
        return MultiDataFormatterYearAndFrom2(country_bot, year_bot, category_relation_mapping=mapping)

    def test_resolve_relation_label_from(self, bot_with_mapping):
        """Test resolve_relation_label with 'from' relation."""
        result = bot_with_mapping.resolve_relation_label("Writers from Yemen", "كتاب")
        assert result == "كتاب من"

    def test_resolve_relation_label_in(self, bot_with_mapping):
        """Test resolve_relation_label with 'in' relation."""
        result = bot_with_mapping.resolve_relation_label("People in Germany", "أشخاص")
        assert result == "أشخاص في"

    def test_resolve_relation_label_no_match(self, bot_with_mapping):
        """Test resolve_relation_label when no relation matches."""
        result = bot_with_mapping.resolve_relation_label("German writers", "كتاب")
        assert result == "كتاب"

    def test_resolve_relation_label_empty_base_label(self, bot_with_mapping):
        """Test resolve_relation_label with empty base_label."""
        result = bot_with_mapping.resolve_relation_label("Writers from Yemen", "")
        assert result == ""

    def test_resolve_relation_label_empty_category(self, bot_with_mapping):
        """Test resolve_relation_label with empty category."""
        result = bot_with_mapping.resolve_relation_label("", "كتاب")
        assert result == "كتاب"

    def test_resolve_relation_label_already_has_relation(self, bot_with_mapping):
        """Test resolve_relation_label when base_label already ends with relation."""
        result = bot_with_mapping.resolve_relation_label("Writers from Yemen", "كتاب من")
        assert result == "كتاب من"

    def test_resolve_relation_label_relation_in_middle(self, bot_with_mapping):
        """Test resolve_relation_label when relation is in the middle of base_label."""
        result = bot_with_mapping.resolve_relation_label("Writers from Yemen", "كتاب من اليمن")
        assert result == "كتاب من اليمن"


class TestMultiDataFormatterYearAndFrom2GetRelationMapping:
    """Tests for MultiDataFormatterYearAndFrom2.get_relation_mapping method."""

    def test_get_relation_mapping_empty(self):
        """Test get_relation_mapping with empty mapping."""
        country_bot = FormatDataFrom(
            formatted_data={},
            key_placeholder="{country1}",
            value_placeholder="{country1}",
            search_callback=lambda x: "",
            match_key_callback=lambda x: "",
        )
        year_bot = FormatDataFrom(
            formatted_data={},
            key_placeholder="{year1}",
            value_placeholder="{year1}",
            search_callback=lambda x: "",
            match_key_callback=lambda x: "",
        )

        bot = MultiDataFormatterYearAndFrom2(country_bot, year_bot)

        assert bot.get_relation_mapping() == {}

    def test_get_relation_mapping_with_data(self):
        """Test get_relation_mapping with populated mapping."""
        country_bot = FormatDataFrom(
            formatted_data={},
            key_placeholder="{country1}",
            value_placeholder="{country1}",
            search_callback=lambda x: "",
            match_key_callback=lambda x: "",
        )
        year_bot = FormatDataFrom(
            formatted_data={},
            key_placeholder="{year1}",
            value_placeholder="{year1}",
            search_callback=lambda x: "",
            match_key_callback=lambda x: "",
        )
        mapping = {"from": "من", "in": "في"}

        bot = MultiDataFormatterYearAndFrom2(country_bot, year_bot, category_relation_mapping=mapping)

        result = bot.get_relation_mapping()
        assert "from" in result
        assert "in" in result
        assert result["from"] == "من"
        assert result["in"] == "في"



"""
Pattern-based category resolvers for the ArWikiCats project.
This package provides resolvers that use complex regex patterns to match
and translate categories with structured temporal or nationality components.
"""

import functools
import logging

from . import (
    country_nat_pattern,
    country_time_pattern,
    nat_males_pattern,
    time_patterns_resolvers,
)

logger = logging.getLogger(__name__)


@functools.lru_cache(maxsize=50000)
def all_patterns_resolvers(category: str) -> str:
    """
    Resolve a category name into a canonical label using pattern-based rules.

    Parameters:
        category (str): The category string to be analyzed and mapped.

    Returns:
        str: The resolved category label, or an empty string if no pattern matches.
    """
    logger.debug(f">> : {category}")
    category_lab = (
        country_time_pattern.resolve_country_time_pattern(category)
        or nat_males_pattern.resolve_nat_males_pattern(category)
        or time_patterns_resolvers.resolve_lab_from_years_patterns(category)
        or country_nat_pattern.resolve_country_nat_pattern(category)
        or ""
    )
    logger.debug(f"<< : {category} => {category_lab}")
    return category_lab



#!/usr/bin/python3
"""

TODO:
    - add formated_data from ArWikiCats/translations/tv/films_mslslat.py

"""

import functools
import logging
from typing import Dict

from ...translations import (
    Nat_women,
    film_keys_for_female,
)
from ...translations_formats import (
    MultiDataFormatterBase,
    format_films_country_data,
    format_multi_data,
)

logger = logging.getLogger(__name__)


def _build_television_cao() -> tuple[Dict[str, str], Dict[str, str]]:
    """
    Construct two translation mapping dictionaries for television and film category keys: one that includes nationality placeholders and one without.

    The first dictionary maps English pattern keys (which may include `{nat_en}` and `{film_key}`) to Arabic translation templates (which may include `{nat_ar}` and `{film_ar}`). The second dictionary provides equivalent mappings that omit nationality placeholders and may include `{film_ar}` in templates.

    Returns:
        films_key_cao (Dict[str, str]): Mapping of English pattern keys with optional `{nat_en}`/`{film_key}` to Arabic templates containing optional `{nat_ar}`/`{film_ar}`.
        data_no_nats (Dict[str, str]): Mapping of English pattern keys without nationality placeholders to Arabic templates that may include `{film_ar}`.
    """
    data = {}
    data_no_nats = {}

    # Base TV keys with common suffixes
    for suffix, arabic_suffix in [
        ("characters", "شخصيات"),
        ("title cards", "بطاقات عنوان"),
        ("video covers", "أغلفة فيديو"),
        ("posters", "ملصقات"),
        ("images", "صور"),
    ]:
        data_no_nats.update(
            {
                f"{{film_key}} {suffix}": f"{arabic_suffix} {{film_ar}}",
            }
        )
        data.update(
            {
                f"{{nat_en}} {suffix}": f"{arabic_suffix} {{nat_ar}}",
                f"{{nat_en}} {{film_key}} {suffix}": f"{arabic_suffix} {{film_ar}} {{nat_ar}}",
            }
        )

    # Genre-based categories
    # ArWikiCats/jsons/media/Films_key_For_nat.json
    genre_categories = {
        # "fiction": "خيال",
        "film series": "سلاسل أفلام",
        "webcomics": "ويب كومكس",
        "anime and manga": "أنمي ومانغا",
        "compilation albums": "ألبومات تجميعية",
        "folk albums": "ألبومات فلكلورية",
        "classical albums": "ألبومات كلاسيكية",
        "comedy albums": "ألبومات كوميدية",
        "mixtape albums": "ألبومات ميكستايب",
        "soundtracks": "موسيقى تصويرية",
        "terminology": "مصطلحات",
        "series": "مسلسلات",
        "television series": "مسلسلات تلفزيونية",
        "television episodes": "حلقات تلفزيونية",
        "television programs": "برامج تلفزيونية",
        "television programmes": "برامج تلفزيونية",
        "groups": "مجموعات",
        "novellas": "روايات قصيرة",
        "novels": "روايات",
        "films": "أفلام",
        "comic strips": "شرائط كومكس",
        "comics": "قصص مصورة",
        "television shows": "عروض تلفزيونية",
        "television films": "أفلام تلفزيونية",
        "teams": "فرق",
        "television characters": "شخصيات تلفزيونية",
        "video games": "ألعاب فيديو",
        "web series": "مسلسلات ويب",
        "film characters": "شخصيات أفلام",
        "games": "ألعاب",
        "soap opera": "مسلسلات طويلة",
        "television news": "أخبار تلفزيونية",
        "miniseries": "مسلسلات قصيرة",
        "television miniseries": "مسلسلات قصيرة تلفزيونية",
    }

    genre_categories_skip_it = {
        "film characters",
        "series",
        "games",
    }

    # Standard categories
    for suffix, arabic_base in genre_categories.items():
        # Base TV keys with common suffixes
        for sub_suffix, arabic_sub_suffix in [
            ("characters", "شخصيات"),
            ("title cards", "بطاقات عنوان"),
            ("video covers", "أغلفة فيديو"),
            ("posters", "ملصقات"),
            ("images", "صور"),
        ]:
            data_no_nats.update(
                {
                    f"{suffix} {sub_suffix}": f"{arabic_sub_suffix} {arabic_base}",
                    f"{{film_key}} {suffix} {sub_suffix}": f"{arabic_sub_suffix} {arabic_base} {{film_ar}}",
                }
            )
            data.update(
                {
                    f"{{nat_en}} {suffix} {sub_suffix}": f"{arabic_sub_suffix} {arabic_base} {{nat_ar}}",
                    f"{{nat_en}} {{film_key}} {suffix} {sub_suffix}": f"{arabic_sub_suffix} {arabic_base} {{film_ar}} {{nat_ar}}",
                }
            )

        data_no_nats.update(
            {
                f"{suffix}": f"{arabic_base}",
                f"television {suffix}": f"{arabic_base} تلفزيونية",
                f"{{film_key}} {suffix}": f"{arabic_base} {{film_ar}}",
                f"children's-animated-superhero {suffix}": f"{arabic_base} رسوم متحركة أبطال خارقين للأطفال",
                f"children's-animated-adventure-television {suffix}": f"{arabic_base} مغامرات رسوم متحركة تلفزيونية للأطفال",
            }
        )

        # NOTE: we use genre_categories_skip_it because next line makes errors like:
        # "Category:Golf at 2022 Asian Games": "تصنيف:الغولف في ألعاب آسيوية في 2022",
        if suffix not in genre_categories_skip_it:
            data[f"{{nat_en}} {suffix}"] = f"{arabic_base} {{nat_ar}}"

        data.update(
            {
                f"{{nat_en}} {{film_key}} {suffix}": f"{arabic_base} {{film_ar}} {{nat_ar}}",
                f"{{nat_en}} children's-animated-superhero {suffix}": f"{arabic_base} رسوم متحركة أبطال خارقين {{nat_ar}} للأطفال",
                f"{{nat_en}} children's-animated-adventure-television {suffix}": f"{arabic_base} مغامرات رسوم متحركة تلفزيونية {{nat_ar}} للأطفال",
            }
        )

    return data, data_no_nats


@functools.lru_cache(maxsize=1)
def _make_bot() -> MultiDataFormatterBase:
    # NOTE: keys with non-patterns should be added to populate_film_patterns()
    # Template data with both nationality and sport placeholders
    """
    Create and configure formatter bots for film and television category translations.

    Builds and merges formatted pattern data (including television CAO entries and film-key mappings),
    prepares nationality and film-key lookup lists, and generates two formatter instances:
    - `double_bot`: a combined formatter populated with country+film patterns and additional adjustments.
    - `bot`: a multi-data formatter built from the same inputs.

    This function also updates `double_bot.other_bot` to set the `put_label_last` label ordering.

    Returns:
        tuple: `(double_bot, bot)` where `double_bot` is the combined MultiDataFormatterBase with populated film-country patterns and `bot` is an additional MultiDataFormatterBase built from the same formatted data.
    """
    formatted_data = {
        # "{nat_en} films": "أفلام {nat_ar}", #  [2000s American films] : "تصنيف:أفلام أمريكية في عقد 2000",
        "{nat_en} films": "أفلام {nat_ar}",
        "remakes of {nat_en} films": "أفلام {nat_ar} معاد إنتاجها",
        # "Category:yemeni action Teen superhero films" : "تصنيف:أفلام حركة مراهقة يمنية أبطال خارقين",
        "{nat_en} television episodes": "حلقات تلفزيونية {nat_ar}",
        "{nat_en} television series": "مسلسلات تلفزيونية {nat_ar}",
        "remakes of {nat_en} television series": "مسلسلات تلفزيونية {nat_ar} معاد إنتاجها",
        "{nat_en} television-seasons": "مواسم تلفزيونية {nat_ar}",
        "{nat_en} television seasons": "مواسم تلفزيونية {nat_ar}",
        "{nat_en} {film_key} television-seasons": "مواسم تلفزيونية {film_ar} {nat_ar}",
        "{nat_en} {film_key} television seasons": "مواسم تلفزيونية {film_ar} {nat_ar}",
        "{nat_en} {film_key} television series": "مسلسلات تلفزيونية {film_ar} {nat_ar}",
        "{nat_en} {film_key} filmszz": "أفلام {film_ar} {nat_ar}",
        "{nat_en} {film_key} films": "أفلام {film_ar} {nat_ar}",
        "remakes of {nat_en} {film_key} films": "أفلام {film_ar} {nat_ar} معاد إنتاجها",
        "{nat_en} {film_key} television commercials": "إعلانات تجارية تلفزيونية {film_ar} {nat_ar}",
        "{nat_en} animated television films": "أفلام رسوم متحركة تلفزيونية {nat_ar}",
        "{nat_en} animated television series": "مسلسلات رسوم متحركة تلفزيونية {nat_ar}",
    }

    _data, data_no_nats = _build_television_cao()

    formatted_data.update(_data)

    other_formatted_data = {
        "{film_key} films": "أفلام {film_ar}",
        "remakes of {film_key} films": "أفلام {film_ar} معاد إنتاجها",
        # "Category:action Teen superhero films" : "تصنيف:أفلام حركة مراهقة أبطال خارقين",
        "{film_key} television commercials": "إعلانات تجارية تلفزيونية {film_ar}",
        "animated television films": "أفلام رسوم متحركة تلفزيونية",
        "animated television series": "مسلسلات رسوم متحركة تلفزيونية",
    }
    other_formatted_data.update(data_no_nats)

    # film_keys_for_female
    data_list2 = {
        "action comedy": "حركة كوميدية",
        "action thriller": "إثارة حركة",
        "action": "حركة",
        "drama": "درامية",
        "upcoming": "قادمة",
        "horror": "رعب",
        "black-and-white": "أبيض وأسود",
        "psychological horror": "رعب نفسي",
    }

    put_label_last = {
        "low-budget",
        "supernatural",
        "christmas",
        "lgbtq-related",
        "upcoming",
    }

    data_list2 = dict(film_keys_for_female)
    data_list2.pop("television", None)

    # data_list2.pop("superhero", None)
    data_list2["superhero"] = "أبطال خارقين"

    double_bot = format_films_country_data(
        formatted_data=formatted_data,
        data_list=Nat_women,
        key_placeholder="{nat_en}",
        value_placeholder="{nat_ar}",
        data_list2=data_list2,
        key2_placeholder="{film_key}",
        value2_placeholder="{film_ar}",
        text_after="",
        text_before="",
        other_formatted_data=other_formatted_data,
    )

    double_bot.other_bot.update_put_label_last(put_label_last)
    bot = format_multi_data(
        formatted_data=formatted_data,
        data_list=Nat_women,
        key_placeholder="{nat_en}",
        value_placeholder="{nat_ar}",
        data_list2=data_list2,
        key2_placeholder="{film_key}",
        value2_placeholder="{film_ar}",
        text_after="",
        text_before="",
        other_formatted_data=other_formatted_data,
    )
    return double_bot, bot


def fix_keys(category: str) -> str:
    """
    Normalize a category key and apply specific known corrections.

    Parameters:
        category (str): Original category key string (may include a leading "category:" prefix).

    Returns:
        str: Normalized category key in lowercase with the "category:" prefix removed and specific fixes applied
        (e.g., "saudi arabian" -> "saudiarabian", "children's animated adventure television" ->
        "children's-animated-adventure-television").
    """
    # normalized_text = category.lower().replace("category:", " ").strip()
    fixes = {
        "saudi arabian": "saudiarabian",
        # "animated television": "animated-television",
        "children's animated adventure television": "children's-animated-adventure-television",
        "children's animated superhero": "children's-animated-superhero",
    }
    category = category.lower().strip()

    for old, new in fixes.items():
        category = category.replace(old, new)

    return category


@functools.lru_cache(maxsize=10000)
def _get_films_key_tyty_new(text: str) -> str:
    """
    Resolve a films category key from free-form category text.

    Parameters:
        text (str): Free-form category or country identifier text to match against known film/television keys.

    Returns:
        resolved_key (str): The matched films key, or an empty string if no match is found.
    """
    normalized_text = fix_keys(text)
    logger.debug(f"<<yellow>> start {normalized_text=}")
    double_bot, bot = _make_bot()

    result = bot.search_all(normalized_text) or double_bot.search_all(normalized_text)
    logger.info(f"<<yellow>> end {normalized_text=}, {result=}")
    return result


@functools.lru_cache(maxsize=10000)
def get_films_key_tyty_new(text: str) -> str:
    """
    Resolve a films/television category key from an input text.

    Parameters:
        text (str): Category or country identifier text to analyze.

    Returns:
        str: Resolved category key, or an empty string if no match is found.
    """
    # return ""
    return _get_films_key_tyty_new(text)



""" """

import pytest

from ArWikiCats import resolve_label_ar
from utils.dump_runner import make_dump_test_name_data

data_0 = {
    "Suicides by Jews during Holocaust": "منتحرون بواسطة يهود خلال هولوكوست",
    "Persecution of Christians by Hindus": "اضطهاد مسيحيون بواسطة هندوس",
    "Persecution of Christians by Jews": "اضطهاد مسيحيون بواسطة يهود",
    "Persecution of Christians by Muslims": "اضطهاد مسيحيون بواسطة مسلمون",
    "Persecution of Hindus by Muslims": "اضطهاد هندوس بواسطة مسلمون",
    "Persecution of Muslims by Christians": "اضطهاد مسلمون بواسطة مسيحيون",
    "Persecution of Muslims by Jews": "اضطهاد مسلمون بواسطة يهود",
    "Persecution of Yazidis by Muslims": "اضطهاد يزيديون بواسطة مسلمون",
}

data_1 = {
    "Water resource management in Netherlands": "إدارة الموارد المائية في هولندا",
    "Sports ministers of Comoros": "وزراء رياضة جزر القمر",
    "Sports ministers of Nauru": "وزراء رياضة ناورو",
    "Sports ministers of Sri Lanka": "وزراء رياضة سريلانكا",
    "National sports teams of Saint Helena": "منتخبات رياضية وطنية في سانت هيلانة",
    "November 2010 crimes": "جرائم نوفمبر 2010",
    "October 2010 crimes": "جرائم أكتوبر 2010",
    "July 2010 crimes": "جرائم يوليو 2010",
    "June 2010 crimes": "جرائم يونيو 2010",
    "March 2010 crimes": "جرائم مارس 2010",
    "February 2010 crimes": "جرائم فبراير 2010",
    "February 2010 events by country": "أحداث فبراير 2010 حسب البلد",
    "August 2010 crimes": "جرائم أغسطس 2010",
    "Businesspeople in aviation by nationality": "شخصيات أعمال في طيران حسب الجنسية",
    "2010 in motorsport by country": "رياضة المحركات في 2010 حسب البلد",
    "2010s in international relations": "علاقات دولية في عقد 2010",
    "2010s in law": "القانون في عقد 2010",
    "2010s introductions": "استحداثات عقد 2010",
    "2010s meteorology": "الأرصاد الجوية في عقد 2010",
    "2010s murders": "جرائم قتل في عقد 2010",
    "2010s natural disasters": "كوارث طبيعية في عقد 2010",
    "2010s non-fiction books": "كتب غير خيالية عقد 2010",
    "2010s paintings": "لوحات عقد 2010",
    "2010s plays": "مسرحيات عقد 2010",
    "2010s poems": "قصائد عقد 2010",
    "2010s sculptures": "منحوتات عقد 2010",
    "2010s ships": "سفن عقد 2010",
    "2010s short stories": "قصص قصيرة عقد 2010",
    "2010s songs": "أغاني عقد 2010",
    "2010s treaties": "معاهدات في عقد 2010",
    "2010s works": "أعمال عقد 2010",
    "21st-century BC births": "مواليد القرن 21 ق م",
    "220 BC births": "مواليد 220 ق م",
    "260 births": "مواليد 260",
    "276 births": "مواليد 276",
    "278 deaths": "وفيات 278",
    "298 deaths": "وفيات 298",
    "365 BC deaths": "وفيات 365 ق م",
    "448 births": "مواليد 448",
    "498 births": "مواليد 498",
    "501 deaths": "وفيات 501",
    "540s births": "مواليد عقد 540",
    "647 births": "مواليد 647",
    "672 deaths": "وفيات 672",
    "721 births": "مواليد 721",
    "730s births": "مواليد عقد 730",
    "808 births": "مواليد 808",
    "921 births": "مواليد 921",
    "969 births": "مواليد 969",
}


TEMPORAL_CASES = [
    ("test_resolve_label_ar_1", data_1),
]


@pytest.mark.parametrize("category, expected", data_1.items(), ids=data_1.keys())
@pytest.mark.fast
def test_resolve_label_ar_1(category: str, expected: str) -> None:
    label = resolve_label_ar(category)
    assert label == expected


test_dump_all = make_dump_test_name_data(TEMPORAL_CASES, resolve_label_ar, run_same=False)



#!/usr/bin/python3
"""JSON file loading utilities for translation data.

This module provides cached JSON file loading functions for the translation
dictionaries used throughout ArWikiCats.
"""

import functools
import json
import logging
from pathlib import Path
from typing import Any, Dict, List

logger = logging.getLogger(__name__)

Dir2 = Path(__file__).parent.parent.parent


def _build_json_path(relative_path: str) -> Path:
    """Return the full path to a JSON file under ``jsons``.

    The helper accepts either bare filenames (``"example"``) or paths that
    include nested folders (``"geography/us_counties"``). When the provided
    path does not include an extension, ``.json`` is appended automatically.
    """
    path = Path(relative_path)
    if path.suffix != ".json":
        path = path.with_suffix(".json")
    return Dir2 / "jsons" / path


@functools.lru_cache(maxsize=128)
def open_json_file(file_path: str = "") -> Dict[str, Any] | List[Any]:
    """Open a JSON resource from the bundled ``jsons`` directory by name.

    Results are cached to avoid repeated file I/O for the same file.

    Args:
        file_path: Relative path to the JSON file (with or without .json extension).

    Returns:
        The parsed JSON data, or empty dict if the file cannot be loaded.
    """
    if not file_path:
        return {}
    file_path_path = _build_json_path(file_path)
    if not file_path_path.exists():
        logger.warning(f"JSON file not found: {file_path_path}")
        return {}
    try:
        with open(file_path_path, "r", encoding="utf-8") as f:
            return json.load(f)  # type: ignore[no-any-return]
    except (json.JSONDecodeError, OSError) as e:
        logger.error(f"Failed to load JSON from {file_path_path}: {e}")
        return {}



#
import pytest

from ArWikiCats import resolve_label_ar
from utils.dump_runner import make_dump_test_name_data

data_0 = {
    "World War II political leaders": "زعماء الحرب العالمية الثانية",
    "Vanuatu political leader navigational boxes": "صناديق تصفح قادة فانواتو السياسيون",
    "Somaliland political leader navigational boxes": "صناديق تصفح قادة أرض الصومال السياسيون",
    "Republic of the Congo political leader navigational boxes": "صناديق تصفح قادة جمهورية الكونغو السياسيين",
    "Northern Mariana Islands political leader navigational boxes": "صناديق تصفح قادة جزر ماريانا الشمالية السياسيون",
    "Ireland political leader navigational boxes": "قوالب تصفح قادة سياسيين أيرلنديين",
    "European Union political leader navigational boxes": "صناديق تصفح قائد سياسي الاتحاد الأوروبي",
}

data_fast = {
    "Zimbabwe political leader navigational boxes": "صناديق تصفح قادة سياسيون زيمبابويون",
    "Yemen political leader navigational boxes": "صناديق تصفح قادة سياسيون يمنيون",
    "Vietnam political leader navigational boxes": "صناديق تصفح قادة سياسيون فيتناميون",
    "United States political leader templates": "قوالب قادة سياسيون أمريكيون",
    "United States political leader navigational boxes": "صناديق تصفح قادة سياسيون أمريكيون",
    "United Kingdom political leader templates": "قوالب قادة سياسيون بريطانيون",
    "United Kingdom political leader navigational boxes": "صناديق تصفح قادة سياسيون بريطانيون",
    "United Arab Emirates political leader navigational boxes": "صناديق تصفح قادة سياسيون إماراتيون",
    "Ukraine political leader navigational boxes": "صناديق تصفح قادة سياسيون أوكرانيون",
    "Uganda political leader navigational boxes": "صناديق تصفح قادة سياسيون أوغنديون",
    "Turkey political leader navigational boxes": "صناديق تصفح قادة سياسيون أتراك",
    "Tunisia political leader navigational boxes": "صناديق تصفح قادة سياسيون تونسيون",
}

data_slow = {
    "Thailand political leader navigational boxes": "صناديق تصفح قادة سياسيون تايلنديون",
    "Taiwan political leader navigational boxes": "صناديق تصفح قادة سياسيون تايوانيون",
    "Syria political leader navigational boxes": "صناديق تصفح قادة سياسيون سوريون",
    "Sweden political leader navigational boxes": "صناديق تصفح قادة سياسيون سويديون",
    "Suriname political leader navigational boxes": "صناديق تصفح قادة سياسيون سوريناميون",
    "Sudan political leader navigational boxes": "صناديق تصفح قادة سياسيون سودانيون",
    "Spain political leader navigational boxes": "صناديق تصفح قادة سياسيون إسبان",
    "South Sudan political leader navigational boxes": "صناديق تصفح قادة سياسيون سودانيون جنوبيون",
    "South Korea political leader navigational boxes": "صناديق تصفح قادة سياسيون كوريون جنوبيون",
    "South America political leader navigational boxes": "صناديق تصفح قادة سياسيون أمريكيون جنوبيون",
    "Somalia political leader navigational boxes": "صناديق تصفح قادة سياسيون صوماليون",
    "Slovakia political leader navigational boxes": "صناديق تصفح قادة سياسيون سلوفاكيون",
    "Sierra Leone political leader navigational boxes": "صناديق تصفح قادة سياسيون سيراليونيون",
    "Senegal political leader navigational boxes": "صناديق تصفح قادة سياسيون سنغاليون",
    "Saudi Arabia political leader navigational boxes": "صناديق تصفح قادة سياسيون سعوديون",
    "Rwanda political leader navigational boxes": "صناديق تصفح قادة سياسيون روانديون",
    "Romania political leader navigational boxes": "صناديق تصفح قادة سياسيون رومان",
    "Portugal political leader navigational boxes": "صناديق تصفح قادة سياسيون برتغاليون",
    "Philippines political leader navigational boxes": "صناديق تصفح قادة سياسيون فلبينيون",
    "Pakistan political leader navigational boxes": "صناديق تصفح قادة سياسيون باكستانيون",
    "Ottoman Empire political leader navigational boxes": "صناديق تصفح قادة سياسيون عثمانيون",
    "Oman political leader navigational boxes": "صناديق تصفح قادة سياسيون عمانيون",
    "Oceania political leader navigational boxes": "صناديق تصفح قادة سياسيون أوقيانوسيون",
    "North Macedonia political leader navigational boxes": "صناديق تصفح قادة سياسيون مقدونيون شماليون",
    "North America political leader navigational boxes": "صناديق تصفح قادة سياسيون أمريكيون شماليون",
    "Nigeria political leader navigational boxes": "صناديق تصفح قادة سياسيون نيجيريون",
    "Niger political leader navigational boxes": "صناديق تصفح قادة سياسيون نيجريون",
    "New Zealand political leader navigational boxes": "صناديق تصفح قادة سياسيون نيوزيلنديون",
    "Netherlands political leader navigational boxes": "صناديق تصفح قادة سياسيون هولنديون",
    "Namibia political leader navigational boxes": "صناديق تصفح قادة سياسيون ناميبيون",
    "Myanmar political leader navigational boxes": "صناديق تصفح قادة سياسيون ميانماريون",
    "Mozambique political leader navigational boxes": "صناديق تصفح قادة سياسيون موزمبيقيون",
    "Montenegro political leader navigational boxes": "صناديق تصفح قادة سياسيون مونتينيغريون",
    "Moldova political leader navigational boxes": "صناديق تصفح قادة سياسيون مولدوفيون",
    "Middle East political leader navigational boxes": "صناديق تصفح قادة سياسيون شرقيون أوسطيون",
    "Mexico political leader navigational boxes": "صناديق تصفح قادة سياسيون مكسيكيون",
    "Mauritania political leader navigational boxes": "صناديق تصفح قادة سياسيون موريتانيون",
    "Mali political leader navigational boxes": "صناديق تصفح قادة سياسيون ماليون",
    "Madagascar political leader navigational boxes": "صناديق تصفح قادة سياسيون مدغشقريون",
    "Luxembourg political leader navigational boxes": "صناديق تصفح قادة سياسيون لوكسمبورغيون",
    "Lithuania political leader navigational boxes": "صناديق تصفح قادة سياسيون ليتوانيون",
    "Libya political leader navigational boxes": "صناديق تصفح قادة سياسيون ليبيون",
    "Liberia political leader navigational boxes": "صناديق تصفح قادة سياسيون ليبيريون",
    "Lebanon political leader navigational boxes": "صناديق تصفح قادة سياسيون لبنانيون",
    "Laos political leader navigational boxes": "صناديق تصفح قادة سياسيون لاوسيون",
    "Kosovo political leader navigational boxes": "صناديق تصفح قادة سياسيون كوسوفيون",
    "Kazakhstan political leader navigational boxes": "صناديق تصفح قادة سياسيون كازاخستانيون",
    "Jordan political leader navigational boxes": "صناديق تصفح قادة سياسيون أردنيون",
    "Japan political leader navigational boxes": "صناديق تصفح قادة سياسيون يابانيون",
    "Ivory Coast political leader navigational boxes": "صناديق تصفح قادة سياسيون إيفواريون",
    "Italy political leader templates": "قوالب قادة سياسيون إيطاليون",
    "Italy political leader navigational boxes": "صناديق تصفح قادة سياسيون إيطاليون",
    "Israel political leader navigational boxes": "صناديق تصفح قادة سياسيون إسرائيليون",
    "Iraq political leader navigational boxes": "صناديق تصفح قادة سياسيون عراقيون",
    "Iran political leader templates": "قوالب قادة سياسيون إيرانيون",
    "Iran political leader navigational boxes": "صناديق تصفح قادة سياسيون إيرانيون",
    "Indonesia political leader templates": "قوالب قادة سياسيون إندونيسيون",
    "India political leader navigational boxes": "صناديق تصفح قادة سياسيون هنود",
    "Iceland political leader navigational boxes": "صناديق تصفح قادة سياسيون آيسلنديون",
    "Hungary political leader navigational boxes": "صناديق تصفح قادة سياسيون مجريون",
    "Guyana political leader navigational boxes": "صناديق تصفح قادة سياسيون غيانيون",
    "Guinea political leader navigational boxes": "صناديق تصفح قادة سياسيون غينيون",
    "Guinea-Bissau political leader navigational boxes": "صناديق تصفح قادة سياسيون غينيون بيساويون",
    "Guatemala political leader navigational boxes": "صناديق تصفح قادة سياسيون غواتيماليون",
    "Guam political leader navigational boxes": "صناديق تصفح قادة سياسيون غواميون",
    "Greece political leader navigational boxes": "صناديق تصفح قادة سياسيون يونانيون",
    "Germany political leader templates": "قوالب قادة سياسيون ألمان",
    "Germany political leader navigational boxes": "صناديق تصفح قادة سياسيون ألمان",
    "Gabon political leader navigational boxes": "صناديق تصفح قادة سياسيون غابونيون",
    "France political leader navigational boxes": "صناديق تصفح قادة سياسيون فرنسيون",
    "Finland political leader navigational boxes": "صناديق تصفح قادة سياسيون فنلنديون",
    "Fiji political leader navigational boxes": "صناديق تصفح قادة سياسيون فيجيون",
    "Europe political leader navigational boxes": "صناديق تصفح قادة سياسيون أوروبيون",
    "Estonia political leader navigational boxes": "صناديق تصفح قادة سياسيون إستونيون",
    "Egypt political leader navigational boxes": "صناديق تصفح قادة سياسيون مصريون",
    "Democratic Republic of the Congo political leader navigational boxes": "صناديق تصفح قادة سياسيون كونغويون ديمقراطيون",
    "Czechoslovakia political leader navigational boxes": "صناديق تصفح قادة سياسيون تشيكوسلوفاكيون",
    "Czech Republic political leader navigational boxes": "صناديق تصفح قادة سياسيون تشيكيون",
    "Croatia political leader navigational boxes": "صناديق تصفح قادة سياسيون كروات",
    "Colombia political leader navigational boxes": "صناديق تصفح قادة سياسيون كولومبيون",
    "Chad political leader navigational boxes": "صناديق تصفح قادة سياسيون تشاديون",
    "Central African Republic political leader navigational boxes": "صناديق تصفح قادة سياسيون أفارقة أوسطيون",
    "Caribbean political leader navigational boxes": "صناديق تصفح قادة سياسيون كاريبيون",
    "Cape Verde political leader navigational boxes": "صناديق تصفح قادة سياسيون أخضريون",
    "Cameroon political leader navigational boxes": "صناديق تصفح قادة سياسيون كاميرونيون",
    "Cambodia political leader navigational boxes": "صناديق تصفح قادة سياسيون كمبوديون",
    "Burundi political leader navigational boxes": "صناديق تصفح قادة سياسيون بورونديون",
    "Burkina Faso political leader navigational boxes": "صناديق تصفح قادة سياسيون بوركينابيون",
    "Bulgaria political leader navigational boxes": "صناديق تصفح قادة سياسيون بلغاريون",
    "Brazil political leader navigational boxes": "صناديق تصفح قادة سياسيون برازيليون",
    "Botswana political leader navigational boxes": "صناديق تصفح قادة سياسيون بوتسوانيون",
    "Bosnia and Herzegovina political leader navigational boxes": "صناديق تصفح قادة سياسيون بوسنيون",
    "Benin political leader navigational boxes": "صناديق تصفح قادة سياسيون بنينيون",
    "Bangladesh political leader navigational boxes": "صناديق تصفح قادة سياسيون بنغلاديشيون",
    "Azerbaijan political leader navigational boxes": "صناديق تصفح قادة سياسيون أذربيجانيون",
    "Austria political leader navigational boxes": "صناديق تصفح قادة سياسيون نمساويون",
    "Asia political leader navigational boxes": "صناديق تصفح قادة سياسيون آسيويون",
    "Armenia political leader navigational boxes": "صناديق تصفح قادة سياسيون أرمن",
    "Argentina political leader navigational boxes": "صناديق تصفح قادة سياسيون أرجنتينيون",
    "Angola political leader navigational boxes": "صناديق تصفح قادة سياسيون أنغوليون",
    "Algeria political leader navigational boxes": "صناديق تصفح قادة سياسيون جزائريون",
    "Albania political leader navigational boxes": "صناديق تصفح قادة سياسيون ألبان",
    "Africa political leader navigational boxes": "صناديق تصفح قادة سياسيون أفارقة",
    "Afghanistan political leader navigational boxes": "صناديق تصفح قادة سياسيون أفغان",
}

to_test = [
    ("test_political_leader_1", data_fast),
    ("test_political_leader_slow", data_slow),
]


@pytest.mark.parametrize("category, expected", data_fast.items(), ids=data_fast.keys())
@pytest.mark.fast
def test_political_leader_1(category: str, expected: str) -> None:
    assert resolve_label_ar(category) == expected


@pytest.mark.parametrize("category, expected", data_slow.items(), ids=data_slow.keys())
@pytest.mark.slow
def test_political_leader_slow(category: str, expected: str) -> None:
    assert resolve_label_ar(category) == expected


test_dump_all = make_dump_test_name_data(to_test, resolve_label_ar, run_same=True)



#!/usr/bin/python3
"""
Separator-Based Resolver Module

This module provides functionality to translate English category names
into Arabic labels by applying separator-based resolution strategies.

It is extracted from the legacy circular_dependency package to break the import cycle.
This module imports from arabic_label_builder, which in turn imports from country_resolver.
This creates a proper DAG of imports with no cycles.
"""

from __future__ import annotations

import functools
import logging
import re

from ...format_bots.relation_mapping import translation_category_relations
from ...utils import get_relation_word
from .arabic_label_builder import find_ar_label

logger = logging.getLogger(__name__)

en_literes = "[a-z]"


@functools.lru_cache(maxsize=10000)
def work_separator_names(
    category: str,
) -> str:
    """Process categories that contain relational words (separator).

    This function extracts relational words from categories and uses them
    to find appropriate Arabic labels.

    Args:
        category: The category string to process

    Returns:
        The associated Arabic label if found, otherwise an empty string.
    """
    separator, separator_name = get_relation_word(category, translation_category_relations)

    if not separator:
        return ""

    logger.info(f'<<lightblue>>>>>> : separator:"{separator_name}":"{separator}" in category ')
    arlabel = find_ar_label(category, separator, cate_test=category)

    if not arlabel:
        return ""

    # Check if the result contains Arabic characters
    if re.sub(en_literes, "", arlabel, flags=re.IGNORECASE) != arlabel:
        arlabel = ""

    logger.info(f">>>> <<lightyellow>> {arlabel=}")

    return arlabel


__all__ = [
    "work_separator_names",
]



"""
Double-key template-driven translation model (v2).
This module provides the FormatDataDoubleV2 class, which handles categories
where two adjacent keys from a data list appear together.
"""

import functools
import logging
import re
from typing import Dict, Optional, Union

from ..DataModel.model_data_base import FormatDataBase

logger = logging.getLogger(__name__)


class FormatDataDoubleV2(FormatDataBase):
    """
    Handles double-key template-driven category translations.

    This class extends FormatDataBase to handle categories where two adjacent
    keys from the data_list appear together (e.g., "action drama films").
    It can match both single keys and pairs of keys, combining their Arabic
    labels in the correct order, with options to sort labels or place specific
    labels last.

    Attributes:
        formatted_data (Dict[str, str]): Template patterns mapping English patterns to Arabic templates.
        data_list (Dict[str, Union[str, Dict[str, str]]]): Key-to-Arabic-label mappings for replacements.
        key_placeholder (str): Placeholder used in formatted_data keys (default: "xoxo").
        text_after (str): Text to append after the formatted label (default: "").
        text_before (str): Text to prepend before the formatted label (default: "").
        splitter (str): Separator used in patterns for splitting keys (default: " ").
        ar_joiner (str): Joiner for combining Arabic labels (default: " ").
        sort_ar_labels (bool): Whether to sort Arabic labels alphabetically (default: False).
        keys_to_split (dict): Cache mapping combined keys to their component parts.
        put_label_last (set[str] | list[str]): Keys whose labels should appear last in combinations.
        search_multi_cache (dict): Cache for combined label lookups.
        alternation (str): Regex alternation string for keys.
        pattern (re.Pattern): Regex pattern for matching single keys.
        pattern_double (re.Pattern): Regex pattern for matching two adjacent keys.

    Example:
        >>> data_list = {
        ...     "action": {"genre_label": "أكشن"},
        ...     "drama": {"genre_label": "دراما"},
        ... }
        >>> bot = FormatDataDoubleV2(
        ...     formatted_data={"{genre} films": "أفلام {genre_label}"},
        ...     data_list=data_list,
        ...     key_placeholder="{genre}",
        ... )
        >>> bot.search("action drama films")
        'أفلام أكشن دراما'
    """

    def __init__(
        self,
        formatted_data: Dict[str, str],
        data_list: Dict[str, Union[str, Dict[str, str]]],
        key_placeholder: str = "xoxo",
        text_after: str = "",
        text_before: str = "",
        splitter: str = " ",
        ar_joiner: str = " ",
        sort_ar_labels: bool = False,
        log_multi_cache: bool = True,
    ):
        """
        Initialize FormatDataDoubleV2 and prepare caches, regex patterns, and configuration used for single- and double-key template matching and Arabic label assembly.

        Parameters:
            formatted_data: Mapping of English patterns to Arabic templates used for exact and template-based matches.
            data_list: Mapping of keys to Arabic labels or dicts of attribute-specific Arabic labels.
            key_placeholder: Placeholder token used inside templates for key substitution.
            text_after: Text appended to the final generated label.
            text_before: Text prepended to the final generated label.
            splitter: Separator expected between two adjacent keys in source categories (used when building double-key regex).
            ar_joiner: Joiner used to combine two Arabic labels when forming a composite label.
            sort_ar_labels: If True, sort the two Arabic labels alphabetically before joining.
            log_multi_cache: If True, enable caching/logging of multi-key composite label lookups.

        Side effects:
            - Initializes internal caches: keys_to_split, search_multi_cache, and put_label_last.
            - Builds regex alternation and compiles single- and double-key matching patterns (pattern, pattern_double).
        """
        super().__init__(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder=key_placeholder,
            text_after=text_after,
            text_before=text_before,
        )
        self.log_multi_cache = log_multi_cache
        self.sort_ar_labels = sort_ar_labels
        self.keys_to_split = {}
        self.put_label_last = {}
        self.search_multi_cache = {}
        self.splitter = splitter or " "
        self.ar_joiner = ar_joiner or " "

        self.alternation: str = self.create_alternation()
        self.pattern = self.keys_to_pattern()

        self.pattern_double = self.keys_to_pattern_double()

    def update_put_label_last(self, data: list[str] | set[str]) -> None:
        """Update the set of keys whose labels should be placed last in combinations."""
        self.put_label_last = data

    def _search(self, category: str) -> str:
        """
        Resolve a category string to its final formatted label using single- or double-key matching and template substitution.

        Parameters:
            category (str): The input category to match against single-key or adjacent double-key patterns.

        Returns:
            str: The formatted label produced by applying the matched template to the resolved key label, or an empty string if no matching key, label, or template is found.
        """
        logger.debug("$$$ start (): ")
        logger.debug(f"++++++++ {self.__class__.__name__} ++++++++ ")

        if self.formatted_data_ci.get(category):
            return self.formatted_data_ci[category]

        sport_key = self.match_key(category)

        if not sport_key:
            logger.debug(f"No sport key matched for {category=}")
            return ""

        sport_label = self.get_key_label(sport_key)
        if not sport_label:
            logger.debug(f'No sport label matched for sport key: "{sport_key}"')
            return ""

        logger.debug(f"sport label: {sport_label=}")

        template_label = self.get_template(sport_key, category)
        if not template_label:
            logger.debug(f'No template label matched for sport key: "{sport_key}" and {category=}')
            return ""

        logger.debug(f"template_label: {template_label=}")

        result = self.apply_pattern_replacement(template_label, sport_label)
        logger.debug(f"[] apply_pattern_replacement: {result=}")

        logger.debug(f"++++++++ end {self.__class__.__name__} ++++++++ ")

        return result

    def keys_to_pattern_double(self) -> Optional[re.Pattern[str]]:
        """
        Create a compiled case-insensitive regex that matches two adjacent keys separated by the configured splitter.

        Returns:
            re.Pattern[str] | None: A compiled regex that matches "key{splitter}key" (case-insensitive) for keys present in the instance, or `None` if there are no keys to match.
        """
        if not self.data_list_ci:
            return None

        if self.alternation is None:
            self.alternation = self.create_alternation()

        data_pattern = rf"(?<!\w)({self.alternation})({self.splitter})({self.alternation})(?!\w)"
        return re.compile(data_pattern, re.I)

    @functools.lru_cache(maxsize=10000)
    def match_key(self, category: str) -> str:
        """
        Determine the canonical lowercased key that corresponds to the given category string.

        Matches the input against known single-key patterns or two-adjacent-key patterns; for two-key matches returns a combined key formed by the two parts and their separator. Leading/trailing and extra interior whitespace are ignored when matching.

        Parameters:
                category (str): Category text to match against known keys.

        Returns:
                matched_key (str): The canonical lowercased key if a match is found, otherwise an empty string.
        """
        if not self.pattern:
            return ""

        # Normalize the category by removing extra spaces
        normalized_category = " ".join(category.split())
        logger.debug(f">!> : {normalized_category=}")

        if self.data_list_ci.get(normalized_category.lower()):
            logger.debug(f">>!!>> : found in data_list_ci {normalized_category=}")
            return normalized_category.lower()

        match = self.pattern_double.search(f" {normalized_category} ")
        if match:
            first_key = match.group(1).lower()
            splitter = match.group(2).lower()
            second_key = match.group(3).lower()
            result = f"{first_key}{splitter}{second_key}"

            logger.debug(f">!> : {first_key=}, {second_key=}")
            logger.debug(f">!> : {result=}")
            self.keys_to_split[result] = [first_key, second_key]
            return result

        match2 = self.pattern.search(f" {normalized_category} ")
        result = match2.group(1).lower() if match2 else ""
        logger.debug(f"==== {result=}")

        return result

    def apply_pattern_replacement(self, template_label: str, sport_label: Union[str, Dict[str, str]]) -> str:
        """
        Substitute placeholder tokens in a template with values from a mapping.

        Parameters:
                template_label (str): Template string containing placeholders in the form `{key}`.
                sport_label (Union[str, Dict[str, str]]): Mapping of placeholder names to replacement strings; if not a dict the template is returned unchanged.

        Returns:
                str: The template with placeholders replaced by their corresponding values, trimmed of surrounding whitespace.
        """
        logger.debug(f"[] : {template_label=}, ")
        logger.debug(f"[] : {sport_label=}, ")

        if not isinstance(sport_label, dict):
            logger.error("===> : sport_label not dict..")
            return template_label

        final_label = template_label

        if isinstance(sport_label, dict):
            for key, val in sport_label.items():
                if isinstance(val, str) and val:
                    final_label = final_label.replace(f"{{{key}}}", val)

        return final_label.strip()

    @functools.lru_cache(maxsize=5000)
    def create_label_from_keys(self, part1: str, part2: str):
        """
        Builds a mapping of Arabic labels for the two-part key combination.

        Constructs a dict whose keys are the union of subkeys found in both parts and whose values are the combined Arabic label for that subkey. Only subkeys that have labels in both parts are included. The produced label normally follows "first_label + ar_joiner + second_label"; if `part1` is present in `put_label_last` and `part2` is not, the order is switched to "second_label + ar_joiner + first_label". If `sort_ar_labels` is True, the two Arabic labels are alphabetically sorted before joining. When `log_multi_cache` is True, the resulting mapping is cached into `search_multi_cache` under the key "{part2} {part1}".

        Returns:
            dict: Mapping from subkey to combined Arabic label when both parts provide dict labels.
            str: Empty string if either part is missing or does not provide a dict of labels.

        if "upcoming" in self.put_label_last we using:
            "أفلام قادمة رعب يمنية instead of "أفلام رعب قادمة يمنية"
        """

        first_label = self.data_list_ci.get(part1)
        second_label = self.data_list_ci.get(part2)

        if not first_label or not second_label:
            logger.debug(f">>> : missing label for {part1=}, {part2=}")
            return ""

        if not isinstance(first_label, dict) or not isinstance(second_label, dict):
            logger.debug(f">>> : non-dict label for {part1=}, {part2=}")
            return ""

        keys_in_2_parts = list(first_label.keys()) + list(second_label.keys())

        logger.debug(f"!!! : found label for {part1=}, {part2=}")

        compound_data = {}

        for key in keys_in_2_parts:
            compound_data[key] = ""
            first_lab = first_label.get(key, "")
            second_lab = second_label.get(key, "")
            if first_lab and second_lab:
                label = f"{first_lab}{self.ar_joiner}{second_lab}"
                # logger.debug(f"!!! : label: {label}")

                if part1 in self.put_label_last and part2 not in self.put_label_last:
                    label = f"{second_lab}{self.ar_joiner}{first_lab}"

                if self.sort_ar_labels:
                    labels_sorted = sorted([first_lab, second_lab])
                    label = self.ar_joiner.join(labels_sorted)
                compound_data[key] = label
        if self.log_multi_cache:
            self.search_multi_cache[f"{part2} {part1}"] = compound_data

        return compound_data

    def get_key_label(self, sport_key: str) -> dict[str, str]:
        """
        Retrieve the Arabic label mapping for the given key, building or fetching combined labels for two-key entries when necessary.

        Returns:
            dict[str, str]: Mapping of attribute keys to Arabic label strings if found; empty string otherwise.
        """
        logger.debug(f"@@ : {sport_key=}")

        result = self.data_list_ci.get(sport_key)
        if result:
            return result

        if self.search_multi_cache.get(sport_key.lower()):
            logger.debug(f"@@ : found in search_multi_cache {sport_key=}")
            return self.search_multi_cache[sport_key.lower()]

        if sport_key in self.keys_to_split:
            part1, part2 = self.keys_to_split[sport_key]
            logger.debug(f"@@ : found in keys_to_split {sport_key=}")
            return self.create_label_from_keys(part1, part2)

        return ""

    def replace_value_placeholder(self, label: str, value: Union[str, Dict[str, str]]) -> str:
        """
        Replace placeholders in a label string with corresponding values from a mapping.

        @param value: Mapping of placeholder names to replacement strings; keys are placeholder names (without braces). If `value` is not a mapping, the label is returned unchanged.
        @returns: The label with each `{key}` replaced by its corresponding string from `value`; unchanged if no replacements apply.
        """
        logger.debug(f"@@ : {label=}, {value=}")

        if not isinstance(value, dict):
            return label

        final_label = label
        for key, val in value.items():
            if isinstance(val, str) and val:
                final_label = final_label.replace(f"{{{key}}}", val)

        return final_label



""" """

import pytest
from load_one_data import dump_diff, dump_same_and_not_same, one_dump_test

from ArWikiCats import resolve_label_ar

fast_data = {
    "Remakes of American films": "أفلام أمريكية معاد إنتاجها",
    "Remakes of Argentine films": "أفلام أرجنتينية معاد إنتاجها",
    "Remakes of Australian films": "أفلام أسترالية معاد إنتاجها",
    "Remakes of Austrian films": "أفلام نمساوية معاد إنتاجها",
    "Remakes of Belgian films": "أفلام بلجيكية معاد إنتاجها",
    "Remakes of Brazilian films": "أفلام برازيلية معاد إنتاجها",
    "Remakes of British films": "أفلام بريطانية معاد إنتاجها",
    "Remakes of Burmese films": "أفلام بورمية معاد إنتاجها",
    "Remakes of Canadian films": "أفلام كندية معاد إنتاجها",
    "Remakes of Chilean films": "أفلام تشيلية معاد إنتاجها",
    "Remakes of Chinese films": "أفلام صينية معاد إنتاجها",
    "Remakes of Danish films": "أفلام دنماركية معاد إنتاجها",
    "Remakes of Dutch films": "أفلام هولندية معاد إنتاجها",
    "Remakes of Finnish films": "أفلام فنلندية معاد إنتاجها",
    "Remakes of French films": "أفلام فرنسية معاد إنتاجها",
    "Remakes of German films": "أفلام ألمانية معاد إنتاجها",
    "Remakes of Hong Kong films": "أفلام هونغ كونغية معاد إنتاجها",
    "Remakes of Hungarian films": "أفلام مجرية معاد إنتاجها",
    "Remakes of Icelandic films": "أفلام آيسلندية معاد إنتاجها",
    "Remakes of Indian films": "أفلام هندية معاد إنتاجها",
    "Remakes of Indian television series": "مسلسلات تلفزيونية هندية معاد إنتاجها",
    "Remakes of Indonesian films": "أفلام إندونيسية معاد إنتاجها",
    "Remakes of Irish films": "أفلام أيرلندية معاد إنتاجها",
    "Remakes of Italian films": "أفلام إيطالية معاد إنتاجها",
    "Remakes of Japanese films": "أفلام يابانية معاد إنتاجها",
    "Remakes of Malaysian films": "أفلام ماليزية معاد إنتاجها",
    "Remakes of Mexican films": "أفلام مكسيكية معاد إنتاجها",
    "Remakes of Norwegian films": "أفلام نرويجية معاد إنتاجها",
    "Remakes of Pakistani films": "أفلام باكستانية معاد إنتاجها",
    "Remakes of Philippine films": "أفلام فلبينية معاد إنتاجها",
    "Remakes of Russian films": "أفلام روسية معاد إنتاجها",
    "Remakes of South Korean films": "أفلام كورية جنوبية معاد إنتاجها",
    "Remakes of Spanish films": "أفلام إسبانية معاد إنتاجها",
    "Remakes of Sri Lankan films": "أفلام سريلانكية معاد إنتاجها",
    "Remakes of Swedish films": "أفلام سويدية معاد إنتاجها",
    "Remakes of Taiwanese films": "أفلام تايوانية معاد إنتاجها",
    "Remakes of Thai films": "أفلام تايلندية معاد إنتاجها",
    "Remakes of Turkish films": "أفلام تركية معاد إنتاجها",
}


@pytest.mark.parametrize("category, expected", fast_data.items(), ids=fast_data.keys())
def test_Keep_it_last_extended(category: str, expected: str) -> None:
    label = resolve_label_ar(category)
    assert label == expected


TEMPORAL_CASES = [
    ("test_Keep_it_last_extended", fast_data),
]


@pytest.mark.parametrize("name,data", TEMPORAL_CASES)
@pytest.mark.dump
def test_Keep_it_last_dump(name: str, data: dict[str, str]) -> None:
    expected, diff_result = one_dump_test(data, resolve_label_ar)
    dump_diff(diff_result, name)
    dump_same_and_not_same(data, diff_result, name)
    assert diff_result == expected, f"Differences found: {len(diff_result):,}, len all :{len(data):,}"



#!/usr/bin/python3
"""
Tests for model_data_base.py module.

This module provides tests for FormatDataBase abstract class which is the foundation
for all single-element category translation formatters.
"""

import pytest

from ArWikiCats.translations_formats.DataModel.model_data_base import FormatDataBase


class TestFormatDataBaseAbstractMethods:
    """Tests for abstract methods that raise NotImplementedError."""

    def test_apply_pattern_replacement_raises(self):
        """Test apply_pattern_replacement raises NotImplementedError in base class."""

        bot = FormatDataBase(
            formatted_data={},
            data_list={},
            key_placeholder="{sport}",
        )
        with pytest.raises(NotImplementedError):
            bot.apply_pattern_replacement("template", "label")

    def test_replace_value_placeholder_raises(self):
        """Test replace_value_placeholder raises NotImplementedError in base class."""

        bot = FormatDataBase(
            formatted_data={},
            data_list={},
            key_placeholder="{sport}",
        )
        with pytest.raises(NotImplementedError):
            bot.replace_value_placeholder("label", "value")



"""
Unit tests for check_bot module.
"""

import pytest

from ArWikiCats.legacy_bots.make_bots.check_bot import (
    add_key_new_players,
    check_key_in_tables,
    check_key_new_players,
    check_key_new_players_n,
    set_tables,
)

# ---------------------------------------------------------------------------
# Tests for check_key_in_tables function
# ---------------------------------------------------------------------------


@pytest.mark.fast
class TestCheckKeyInTables:
    """Tests for the check_key_in_tables function."""

    def test_returns_false_for_empty_tables(self) -> None:
        """Should return False for empty tables list."""
        result = check_key_in_tables("test", [])
        assert result is False

    def test_returns_true_when_key_in_dict(self) -> None:
        """Should return True when key is found in a dict."""
        tables = [{"test": "value"}]
        result = check_key_in_tables("test", tables)
        assert result is True

    def test_returns_true_when_key_in_list(self) -> None:
        """Should return True when key is found in a list."""
        tables = [["test", "other"]]
        result = check_key_in_tables("test", tables)
        assert result is True

    def test_returns_true_when_key_in_set(self) -> None:
        """Should return True when key is found in a set."""
        tables = [{"test", "other"}]
        result = check_key_in_tables("test", tables)
        assert result is True

    def test_returns_false_when_key_not_found(self) -> None:
        """Should return False when key is not found."""
        tables = [{"other": "value"}, ["another"]]
        result = check_key_in_tables("test", tables)
        assert result is False

    def test_case_insensitive_lookup(self) -> None:
        """Should check both original and lowercase keys."""
        tables = [{"test": "value"}]
        # Both original case and lowercase should find the key
        result_uppercase = check_key_in_tables("TEST", tables)
        result_lowercase = check_key_in_tables("test", tables)
        # Lowercase key should always be found in the dict
        assert result_lowercase is True
        # The function should also check lowercase version of the input
        assert result_uppercase is True or result_lowercase is True


# ---------------------------------------------------------------------------
# Tests for check_key_new_players function
# ---------------------------------------------------------------------------


@pytest.mark.fast
class TestCheckKeyNewPlayers:
    """Tests for the check_key_new_players function."""

    def test_returns_boolean(self) -> None:
        """Should return a boolean."""
        result = check_key_new_players("test")
        assert isinstance(result, bool)

    def test_returns_false_for_unknown_key(self) -> None:
        """Should return False for unknown key."""
        result = check_key_new_players("unknown_xyz_key_123456")
        assert result is False

    def test_case_insensitive(self) -> None:
        """Should be case insensitive."""
        result1 = check_key_new_players("TEST")
        result2 = check_key_new_players("test")
        # Both should return the same result
        assert result1 == result2


# ---------------------------------------------------------------------------
# Tests for check_key_new_players_n function
# ---------------------------------------------------------------------------


@pytest.mark.fast
class TestCheckKeyNewPlayersN:
    """Tests for the check_key_new_players_n function."""

    def test_returns_boolean(self) -> None:
        """Should return a boolean."""
        result = check_key_new_players_n("test")
        assert isinstance(result, bool)

    def test_returns_false_for_unknown_key(self) -> None:
        """Should return False for unknown key."""
        result = check_key_new_players_n("unknown_xyz_key_123456")
        assert result is False

    def test_case_insensitive(self) -> None:
        """Should be case insensitive."""
        result1 = check_key_new_players_n("TEST")
        result2 = check_key_new_players_n("test")
        # Both should return the same result
        assert result1 == result2


# ---------------------------------------------------------------------------
# Tests for add_key_new_players function
# ---------------------------------------------------------------------------


@pytest.fixture
def players_new_keys_fixture():
    """Fixture to provide access to players_new_keys and clean up test keys."""
    from ArWikiCats.legacy_bots.make_bots.bot import players_new_keys

    test_keys_added = []
    yield players_new_keys, test_keys_added
    # Cleanup: remove any test keys that were added
    for key in test_keys_added:
        if key in players_new_keys:
            del players_new_keys[key]


@pytest.mark.fast
class TestAddKeyNewPlayers:
    """Tests for the add_key_new_players function."""

    def test_adds_key_to_players_new_keys(self, players_new_keys_fixture) -> None:
        """Should add key to players_new_keys."""
        players_new_keys, test_keys_added = players_new_keys_fixture

        # Use a unique key that won't conflict
        test_key = "test_add_key_12345"
        test_value = "تسمية اختبار"

        # Track this key for cleanup
        test_keys_added.append(test_key.lower())

        add_key_new_players(test_key, test_value, "test_file")
        assert test_key.lower() in players_new_keys
        assert players_new_keys[test_key.lower()] == test_value

    def test_normalizes_key_to_lowercase(self, players_new_keys_fixture) -> None:
        """Should normalize key to lowercase."""
        players_new_keys, test_keys_added = players_new_keys_fixture

        test_key = "TEST_UPPERCASE_KEY_12345"
        test_value = "تسمية"

        # Track this key for cleanup
        test_keys_added.append(test_key.lower())

        add_key_new_players(test_key, test_value, "test_file")
        assert test_key.lower() in players_new_keys
        assert test_key not in players_new_keys  # Original case should not exist


# ---------------------------------------------------------------------------
# Tests for set_tables
# ---------------------------------------------------------------------------


@pytest.mark.fast
class TestSetTables:
    """Tests for the set_tables list."""

    def test_set_tables_is_list(self) -> None:
        """set_tables should be a list."""
        assert isinstance(set_tables, list)

    def test_set_tables_not_empty(self) -> None:
        """set_tables should not be empty."""
        assert len(set_tables) > 0

    def test_set_tables_contains_expected_tables(self) -> None:
        """set_tables should contain expected tables."""
        from ArWikiCats.legacy_bots.make_bots.bot import players_new_keys
        from ArWikiCats.translations import Jobs_new, jobs_mens_data

        assert players_new_keys in set_tables
        assert Jobs_new in set_tables
        assert jobs_mens_data in set_tables



"""Canonical public API for gendered job datasets.

This package aggregates the typed datasets exposed across the refactored job
modules and publishes a stable import surface for downstream callers.  Importers
can rely on :mod:`translations.jobs` to retrieve commonly used mappings without
pulling in individual module internals.
"""

from __future__ import annotations

from .Jobs import Jobs_new, jobs_mens_data, jobs_womens_data
from .Jobs2 import JOBS_2, JOBS_3333
from .jobs_data_basic import NAT_BEFORE_OCC, NAT_BEFORE_OCC_BASE, RELIGIOUS_KEYS_PP
from .jobs_players_list import PLAYERS_TO_MEN_WOMENS_JOBS, SPORT_JOB_VARIANTS
from .jobs_singers import SINGERS_TAB
from .jobs_womens import FEMALE_JOBS_BASE_EXTENDED, short_womens_jobs

__all__ = [
    "SINGERS_TAB",
    "Jobs_new",
    "jobs_mens_data",
    "JOBS_2",
    "JOBS_3333",
    "NAT_BEFORE_OCC",
    "jobs_womens_data",
    "NAT_BEFORE_OCC_BASE",
    "RELIGIOUS_KEYS_PP",
    "PLAYERS_TO_MEN_WOMENS_JOBS",
    "SPORT_JOB_VARIANTS",
    "FEMALE_JOBS_BASE_EXTENDED",
    "short_womens_jobs",
]
